#
# This is a Shiny web application. You can run the application by clicking
# the "Run App" button above.
#
# Find out more about building applications with Shiny here:
#
#    http://shiny.rstudio.com/
#

library(shiny)
library(shinydashboard)
library(fitdistrplus)  # Always load this before dplyr (masks select())
library(dplyr)
library(tidyr)
library(ggplot2)

# Distributions to consider
# - Data are times, so only positive, continuous distributions are valid:
DISTRIBUTIONS <- c("exp", "gamma", "weibull", "lnorm")
DISTRIBUTION_NAMES <- c("Exponential", "Gamma", "Weibull", "Log-normal")
names(DISTRIBUTION_NAMES) <- DISTRIBUTIONS

# Utils:
source("./si_app_utils.R")



## Setup:
incubation <- readRDS("./Incubation.rds") %>% 
	mutate(survLower = inc_survLower,
				 survUpper = inc_survUpper,
				 nimLower = inc_nimLower,
				 nimUpper = inc_nimUpper)
	
morbidity <- readRDS("./Morbidity.rds") %>% 
	mutate(survLower = morb_survLower,
				 survUpper = morb_survUpper,
				 nimLower = morb_nimLower,
				 nimUpper = morb_nimUpper)

data <- list(incubation = incubation,
						 morbidity = morbidity)

data <- lapply(data, function(x) mutate(x, logDose = round(log10(Dose), 2)))


# Main plot settings:
headings <- c(incubation = "Incubation period",
							morbidity = "Clinical period")

xLabels <- c(incubation = "Days since inoculation",
						 morbidity = "Days since symptom onset")

yLabels <- c(incubation = "Proportion still incubating",
						 morbidity = "Proportion surviving")

xmax <- c(incubation = 300,
					morbidity = 50)


## Define UI
ui <- fluidPage(theme = "cerulean", style = "max-width: 1000px;",
   
   titlePanel("Rabies disease progression"),
   
   # Description row
   fluidRow(
   		column(width = 12,
   					 "Distribution of rabies incubation and clinical period durations, derived from experimental within- and",
   					 "cross-species infections. Data collection is described in Mollentze et al. (2020), Virulence mismatches",
   					 "in index hosts shape the outcomes of cross-species transmission, PNAS (in press). See",
						 a("doi:10.5281/zenodo.3746609", href = "https://doi.org/10.5281/zenodo.3746609"),
						 "for raw data.",
   					 HTML("<br><br>"),
   					 em("Observations for individual inoculated animals are shown in grey, arranged vertically by event time to",
   					 	  "approximate a survival curve. Where censoring applies, the range is shown as a horizontal line.",
   					 	  "Interval-censored times are placed using the midpoint between their possible start and end times, ",
   					 	  "while right-censored times are arranged using the time of censoring. Distributions are fit using",
   					 	  HTML("<code>fitdistcens()</code> in the <code>fitdistrplus</code> library.")),

   					 HTML("<br><br><br>")
   		)
   	),
   
   fluidRow(
   		# Controls:
      column(width = 3,
      			 fluidRow(column(width = 12,
      			 								div(style = "margin-left:30px", # Align with main panel of dose plot below
		      			 								 radioButtons("dataset", label = "Dataset:",
		      			 								 						  choices = c("Incubation period" = "incubation",
		      			 								 						 						 "Clinical period" = "morbidity"),
		      			 														  selected = "incubation"),
		      			 								
		      			 								 selectizeInput("sourceSpp", label = "Virus reservoir species:",
		      			 															  choices = NULL,
		      			 															  multiple = TRUE,
		      			 															  options = list(placeholder = "All species")),
		      			 								
		      			 								 selectizeInput("inocSpp", label = "Inoculated species:",
		      			 															  choices = NULL,
		      			 															  multiple = TRUE,
		      			 															  options = list(placeholder = "All species"))
      			 										)
      			 								)
      			 				 ),
      			 
      			 fluidRow(column(width = 12,
      			 								 div(style = "margin-bottom:10px"),
      			 								 div(style = "margin-left:30px", # Align with main panel of plot below
      			 								 		 sliderInput("doseRange",
      			 								 								 HTML("Dose range (log<sub>10</sub> mouse LD<sub>50</sub>):"),
      			 								 								 min = NA,
      			 								 								 max = NA,
      			 								 								 value = c(NA, NA),
      			 								 								 step = 0.01)
      			 								 		),
      			 								 div(
      			 								 		 plotOutput("dosePlot", height = 100)
      			 								 ),
      			 								div(style = "margin-bottom:15px"),
      			 								div(style = "margin-left:30px",
      			 										h5("Selected data:", style = "font-weight:bold; margin-bottom:-0.7em"),
      			 									  tableOutput("selectedDataSummary")
      			 								)
      			 								 ))
      ),
      
      column(width = 1),
      
      # Show stats and plot of the fitted distributions
      column(width = 8,
      			 fluidRow(column(width = 12, align = "center",
      			 								 plotOutput("fitPlot", height = 450)
      			 								)
      			 				 ),
      			 fluidRow(column(width = 12, align = "center",
      			 								 div(style = "margin-bottom:25px"),
      			 								 tableOutput("summaryTable")
      			 								)
      			 				 )
      			 )
   ),
   
   # Allow embedding in a responsive iframe:
   tags$head(
   	tags$script(src = "https://cdnjs.cloudflare.com/ajax/libs/iframe-resizer/3.5.16/iframeResizer.contentWindow.min.js",
   							type = "text/javascript")
   )
)

# Define server logic required to draw a histogram
server <- function(input, output, session) {
	# Select dataset:
	selectedData <- reactive({
		data[[input$dataset]]
	})
	
	# Update available source spp based on selected dataset:
	observe({
		availSourceSpp <- build_spp_list(selectedData(), "SourceSpecies", "SourceOrder")
		updateSelectizeInput(session, "sourceSpp", choices = availSourceSpp)
	})
	
	# Update available inoculated spp based on selected source species:
	sourceSppData <- reactive({
		if (is.null(input$sourceSpp)) 
			selectedData()  # All source species used by default
		else 
			filter(selectedData(), SourceSpecies %in% input$sourceSpp)
	})
	
	observe({
		availInocSpp <- build_spp_list(sourceSppData(), "InoculatedSpecies", "InoculatedOrder")
		updateSelectizeInput(session, "inocSpp", choices = availInocSpp)
	})
	
	# Filter selected inoculated species:
	selectedSppData <- reactive({
		if (is.null(input$inocSpp))
			sourceSppData()  # All spp used by default
		else
			filter(sourceSppData(), InoculatedSpecies %in% input$inocSpp)
	})
	
	
	# Update available dose range:
	observe({
		minVal <- min(sourceSppData()$logDose)
		maxVal <- max(sourceSppData()$logDose)
		
		updateSliderInput(session, "doseRange", 
											min = minVal, max = maxVal, 
											value = c(minVal, maxVal))
	})
	
	# Show distribution of available doses for selected species
	output$dosePlot <- renderPlot({
		validate(
			need(!is.null(input$doseRange), "\tLoading...")
		)
		
		ggplot(selectedSppData(), aes(x = logDose)) +
			geom_histogram(bins = 40) +
			geom_rect(ymin = -Inf, ymax = Inf, xmin = -Inf, xmax = input$doseRange[1], 
								fill = "white", alpha = 0.03) +
			geom_rect(ymin = -Inf, ymax = Inf, xmin = input$doseRange[2], xmax = Inf,
								fill = "white", alpha = 0.03) +
			labs(x = expression(log[10](Dose)), y = "Count") +
			theme_bw()
	})
	
	# Filter by selected dose range
	finalData <- reactive({
		validate(
			need(!is.null(input$doseRange), "\nLoading...")
		)
		
		selectedSppData() %>% 
			filter(logDose >= input$doseRange[1],
						 logDose <= input$doseRange[2])
		})
	
	# Summarise selected data:
	output$selectedDataSummary <- renderTable({
		finalData( ) %>% 
			ungroup() %>% 
			summarise(Publications = length(unique(CitationKey)),
								Experiments = length(unique(ExperimentID)),
								`Unique inocula` = length(unique(InoculumID)),
								Inoculations = n()) %>% 
			gather(key = "", value = "N")
	}, width = "100%")
	
	
	# Fit distributions:
	fits <- reactive({
		fit_all_dists(finalData()$survLower, finalData()$survUpper, dists = DISTRIBUTIONS) %>% 
			add_human_distribution_names(DISTRIBUTION_NAMES)
	})
	
	# Plot fitted distributions:
	output$fitPlot <- renderPlot({
		title = paste0(headings[input$dataset], 
									" (N = ", nrow(finalData()), ")")
		
		generate_plot(fitData = fits(),
									rawData = finalData(),
									xLabel = xLabels[input$dataset],
									yLabel = yLabels[input$dataset],
									title = title,
									xmax = xmax[input$dataset])
	})
	
	
	# Output summary stats:
	output$summaryTable <- renderTable({
		fits() %>% 
			mutate(ParamValue = round(ParamValue, 4),
						 ParamString = paste(Param, ParamValue, sep = "=")) %>% 
			group_by(DistributionName, AIC) %>% 
			summarise(Parameters = paste(ParamString, collapse = ", ")) %>% 
			ungroup() %>% 
			arrange(AIC) %>% 
			mutate(Parameters = if_else(Parameters == "NA=NA", "[fitting failed]", Parameters),
						 AIC = if_else(is.na(AIC), "-", sprintf("%10.2f", AIC))) %>% 
			rename(Distribution = DistributionName)
	})
}

# Run the application 
shinyApp(ui = ui, server = server)

