## Utility functions for rabies serial interval app


# -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Interface functions -----
# -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
build_spp_list <- function(data, sppCol, orderCol, showN = FALSE) {
	# Build a nested list of species names
	validSpp <- data %>% 
		select(Spp = sppCol, Order = orderCol) %>% 
		group_by(Spp, Order) %>% 
		summarise(N = n()) %>% 
		ungroup() %>% 
		filter(N > 0) %>% 
		mutate(SppLabels = Spp)
	
	
	if (showN) {
		validSpp <- validSpp %>% 
			mutate(SppLabels = paste0(Spp, ' (N=', N, ')'))
	}
	
	# Convert to structured list:
	Orders <- validSpp$Order
	result <- sapply(unique(Orders), function (x) validSpp$Spp[Orders == x], simplify = FALSE, USE.NAMES = TRUE)
	
	# Fix names when list contains just one item (selectize ignores the spp name in this case)
	len <- sapply(result, length)
	names(result)[len == 1] <- result[len == 1]
	
	result
}


# -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Fitting functions --------
# -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# (these require fitdistrplus)
fit_dist <- function(distname, data) {
	# Fit a single distribution to the censored data
	#  Data should have columns 'left' and 'right'
	
	baseResult <- data.frame(N = nrow(data),
													 Distribution = distname, 
													 stringsAsFactors = FALSE)
	
	res <- tryCatch({
		fit <- fitdistcens(data, distr = distname)
		cbind(baseResult, 
					AIC = fit$aic,
					Param = names(fit$estimate),
					ParamValue = fit$estimate,
					Comment = NA,
					stringsAsFactors = FALSE)
		
	}, error = function(e) {
		cbind(baseResult, 
					AIC = NA,
					Param = NA,
					ParamValue = NA,
					Comment = as.character(e),
					stringsAsFactors = FALSE)
	})
	
	rownames(res) <- NULL
	res
}


fit_all_dists <- function(leftBound, rightBound, dists) {
	df = data.frame(left = leftBound, 
									right = rightBound)
	
	res <- lapply(dists, fit_dist, data = df)
	bind_rows(res)
}


add_human_distribution_names <- function(fit_data, distribution_names) {
	fit_data %>% 
		mutate(DistributionName = distribution_names[Distribution])
}


# -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Plotting functions --------
# -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
get_rankings <- function(nimlower, nimupper, knownEvent, colourBy, idcolumn = 1) {
	# RANK TIMES TO APPROXIMATE A SURVIVAL CURVE (assuming all individual exprerience an event):
	#  - colourBy can be used to stratify rankings into different curves
	#  - Right censored, known events get ranked by the censor time, 
	#     so they fall amongst the other observed times
	#  - TODO: Right censored, unknown events should get placed at bottom, so they
	#     represent putative survivors
	# USE 'idcolumn' TO SUPPLY ROW IDS IF NEEDED (this function changes the order of rows!)
	df <- data.frame(lower = nimlower,
									 upper = nimupper,
									 knownEvent = knownEvent,
									 col = as.factor(colourBy),
									 Index = idcolumn)
	rankedDf <- data.frame()
	
	for (levl in levels(df$col)) {
		subDf <- df[df$col == levl, ]
		
		rankingVec <- ifelse(is.infinite(subDf$upper),
												 subDf$lower,
												 ifelse(subDf$lower == subDf$upper,
												 			 subDf$lower,  # avoid numerical error caused by division
												 			 (subDf$lower + subDf$upper)/2))
		
		ranks <-  rank(rankingVec, ties.method = 'random')
		rankedY <- 1 - ranks / max(ranks)
		rankedDf <- rbind(rankedDf,
											data.frame(subDf, rankedY = rankedY))
	}
	
	return(rankedDf)
}


get_curve <- function(data, xmax = 300) {
	# Generate a curve for a fitted distribution
	if (any(is.na(data$ParamValue))) 
		return(data.frame(x = NA, y = NA))  # failed fit
	
	stopifnot(all(data$Distribution == data$Distribution[1]))  # Expect only one distribution at a time
	
	densFun <-  paste0('p', data$Distribution[1])
	params <-  as.list(data$ParamValue)
	names(params) <- data$Param
	params$q <- seq(1, xmax, 0.5)
	params$lower.tail <- FALSE
	
	res <- do.call(densFun, params)
	data.frame(x = params$q, 
						 y = res)
}



# Generate a plot for a single dataset:
generate_plot <- function(fitData, rawData, xLabel = 'x', yLabel = 'y', xmax = 300, title = NULL) {
	# Rank raw data:
	rawData <- rawData %>% 
		mutate(Index = row_number())
	
	rankedRawData <- get_rankings(rawData$nimLower, rawData$nimUpper, rawData$knownEvent,
																colourBy = '1',
																idcolumn = rawData$Index) %>%
		select(-knownEvent) %>%
		full_join(rawData, by = 'Index')
	
	#Generate curves for fitted distributions
	plotData <- fitData %>%
		group_by(Distribution, DistributionName) %>%
		do(Curve = get_curve(., xmax)) %>%
		ungroup() %>%
		unnest(cols = Curve)
	
	
	# Plot
	p <- ggplot(rankedRawData, aes(x = lower, y = rankedY)) +
		# Raw data
		geom_segment(aes(xend = upper, yend = rankedY), size = 0.4, colour = '#BBBBBB',
								 data = rankedRawData[!is.infinite(rankedRawData$upper), ]) +
		geom_point(size = 0.8, colour = '#BBBBBB',
							 data = rankedRawData[!is.infinite(rankedRawData$upper) & rankedRawData$lower == rankedRawData$upper, ]) +
		geom_segment(aes(yend = rankedY), xend = Inf, size = 0.4, colour = '#BBBBBB',
								 data = rankedRawData[is.infinite(rankedRawData$upper), ]) +
		
		scale_colour_manual(values = c('#4477AA', '#EE6677', '#228833', '#CCBB44')) +  # Paul Tol's "Bright" colour scheme
		labs(title = title, x = xLabel, y = yLabel, colour = "Distribution") +
		theme_bw() +
		theme(strip.background = element_blank())
	
	# Curves can only be added if there's at least one non-NA value to plot:
	if (!all(is.na(plotData$y))) {
		p <- p +
			geom_line(aes(x = x, y = y, colour = DistributionName), 
								size = 1.5,
								data = plotData)
	}
	
	return(p)
}