
library(dplyr)
library(coda)
library(ggmcmc)
library(purrrlyr)
library(tidyr)
library(tibble)
library(MCMCglmm)
library(ape)
library(rprojroot)

source('Scripts/Utils/utils_mcmcglmm.R')


# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Data ---------------------------------------------------------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
projRoot <- find_rstudio_root_file()
setwd(projRoot)


RemoveWithinSpp <- FALSE  # Within-spp inoculations will be removed below
source('Scripts/Main/PrepareData.R')


TimeTree <- read.tree('Data/External/timetree/Species_timetree.nwk')
TimeTree$tip.label <- sub('_', ' ', TimeTree$tip.label)


# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
## ---- Prepare data: -----------------------------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
dfIncubation <- dfIncubation %>% 
	filter(SourceSpecies != InoculatedSpecies) %>% 
	select(inc_nimLower, inc_nimUpper, RowID)


GlandTitres <- AllData %>% 
	filter(SourceSpecies != InoculatedSpecies) %>% 
	mutate(GlandTitre = if_else(!VirusInSalivaryGlands, 0, GlandTitre)) %>% 
	filter((RabiesConfirmed | RabiesClinicalDiagnosis) & !is.na(GlandTitre)) %>% 
	select(GlandTitre, RowID)

AllData <- AllData %>% 
	select(-inc_nimLower, -inc_nimUpper, -GlandTitre)


dfGlandTitre <- full_join(dfIncubation, GlandTitres, by = 'RowID') %>% 
	left_join(AllData, by = 'RowID') %>%
	mutate(logDose = log10(Dose),
				 DonorIsBat = SourceOrder == 'Chiroptera',
				 
				 logDose_std = standardise_gelman(logDose),
				 InoculationPropDist_std = standardise_gelman(InoculationPropDist),
				 InoculatedKnownHost_cen = center_only(InoculatedKnownHost),
				 DonorIsBat_cen = center_only(DonorIsBat),
				 BodyTempDiff_std = standardise_gelman(BodyTempDiff)) %>% 
	rename(inc = inc_nimLower, 
				 incUpper = inc_nimUpper)

saveRDS(dfGlandTitre, 'Output/dfGlandTitre_includingDerivedVars.rds')



# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
## ---- Model (experiment ID only): -------------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# - Show residual correlation
prior = list(R = list(V = diag(2)*0.02, nu = 4),
						 G = list(G1 = list(V = diag(2), nu = 1, alpha.mu = c(0,0), alpha.V = diag(2)*25^2)))


CorFit <- MCMCglmm(cbind(log(inc), log(incUpper), log1p(GlandTitre)) ~ -1 + trait,
									   random = ~ us(trait):ExperimentID,
										 rcov = ~us(trait):units,
										 prior = prior,
										 family = c('cengaussian', 'gaussian'),
										 data = dfGlandTitre,
										 nitt = 1e6,
										 thin = 100,
										 burnin = 1e5,
										 verbose = FALSE)
saveRDS(CorFit, 'Output/GlandTitres_CorFit_Plain.rds')

summary(CorFit)
plot(CorFit)

# Experiment correlation:
CorFit$VCV[, grepl('ExperimentID', colnames(CorFit$VCV))] %>% 
	posterior.cor() %>% 
	plot()

# Residual correlation:
CorFit$VCV[, grepl('units', colnames(CorFit$VCV))] %>% 
	posterior.cor() %>% 
	plot()


# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
## ---- This cor is (primarily) in the inoculated spp phylogeny: ----------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
prior = list(R = list(V = diag(2)*0.02, nu = 4),
						 G = list(G1 = list(V = diag(2), nu = 1, alpha.mu = c(0,0), alpha.V = diag(2)*25^2),
						 				 G2 = list(V = diag(2), nu = 1, alpha.mu = c(0,0), alpha.V = diag(2)*25^2)))

invA <- inverseA(TimeTree)

CorFit_Inoc <- MCMCglmm(cbind(log(inc), log(incUpper), log1p(GlandTitre)) ~ -1 + trait,
												random = ~ us(trait):ExperimentID + us(trait):InoculatedSpecies,
												rcov = ~us(trait):units,
												prior = prior,
												family = c('cengaussian', 'gaussian'),
												ginverse = list(InoculatedSpecies = invA$Ainv),
												data = dfGlandTitre,
												nitt = 1e6,
												thin = 100,
												burnin = 1e5,
												verbose = FALSE)

saveRDS(CorFit_Inoc, 'Output/GlandTitres_CorFit_InocPhylo.rds')

# Experiment correlation:
CorFit_Inoc$VCV[, grepl('ExperimentID', colnames(CorFit_Inoc$VCV))] %>% 
	posterior.cor() %>% 
	plot()

# Inoc Spp:
CorFit_Inoc$VCV[, grepl('InoculatedSpecies', colnames(CorFit_Inoc$VCV))] %>% 
	posterior.cor() %>% 
	plot()

# Residual correlation:
CorFit_Inoc$VCV[, grepl('units', colnames(CorFit_Inoc$VCV))] %>% 
	posterior.cor() %>% 
	plot()


# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
## ---- Correcting for dose -----------------------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
CorFit_Dose <- MCMCglmm(cbind(log(inc), log(incUpper), log1p(GlandTitre)) ~ -1 + trait + trait:logDose_std,
												random = ~ us(trait):ExperimentID + us(trait):InoculatedSpecies,
												rcov = ~us(trait):units,
												prior = prior,
												family = c('cengaussian', 'gaussian'),
												ginverse = list(InoculatedSpecies = invA$Ainv),
												data = dfGlandTitre,
												nitt = 1e6,
												thin = 100,
												burnin = 1e5,
												verbose = FALSE)

saveRDS(CorFit_Dose, 'Output/GlandTitres_CorFit_InocPhylo+Dose.rds')

# Experiment correlation:
CorFit_Dose$VCV[, grepl('ExperimentID', colnames(CorFit_Dose$VCV))] %>% 
	posterior.cor() %>% 
	plot()

# Inoc Spp:
CorFit_Dose$VCV[, grepl('InoculatedSpecies', colnames(CorFit_Dose$VCV))] %>% 
	posterior.cor() %>% 
	plot()

# Residual correlation:
CorFit_Dose$VCV[, grepl('units', colnames(CorFit_Dose$VCV))] %>% 
	posterior.cor() %>% 
	plot()



# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
## ---- When also including other fixed effects ---------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
CorFitFull <- MCMCglmm(cbind(log(inc), log(incUpper), log1p(GlandTitre)) ~ -1 + trait + 
											 			trait:(logDose_std + InoculationPropDist_std + InoculatedKnownHost_cen*logDose_std +
											 						 	BodyTempDiff_std*logDose_std),
											 
											 random = ~ us(trait):ExperimentID + us(trait):InoculatedSpecies,
											 rcov = ~us(trait):units,
											 prior = prior,
											 family = c('cengaussian', 'gaussian'),
											 ginverse = list(InoculatedSpecies = invA$Ainv),
											 data = dfGlandTitre,
											 nitt = 1e6,
											 thin = 100,
											 burnin = 1e5,
											 verbose = FALSE)

saveRDS(CorFitFull, 'Output/GlandTitres_CorFit_AllVars.rds')

CorFitFull$VCV[, grepl('InoculatedSpecies', colnames(CorFitDose$VCV))] %>% 
	posterior.cor() %>% 
	plot()


CorFitFull$VCV[, grepl('units', colnames(CorFitDose$VCV))] %>% 
	posterior.cor() %>% 
	plot()

