## Fit models to duration data (incubation and clinical periods)

library(dplyr)
library(coda)
library(ggmcmc)
library(purrrlyr)
library(tidyr)
library(tibble)
library(MCMCglmm)
library(ape)
library(rprojroot)

source('Scripts/Utils/utils_mcmcglmm.R')


THIN <- 500
ITERATIONS <- 10000 * THIN
BURNIN <- 0.1 * ITERATIONS
ITERATIONS <- ITERATIONS + BURNIN


# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Data ---------------------------------------------------------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
projRoot <- find_rstudio_root_file()
setwd(projRoot)


RemoveWithinSpp <- FALSE  # Within-spp inoculations will be removed below
source('Scripts/Main/PrepareData.R')

TimeTree <- read.tree('Data/External/timetree/Species_timetree.nwk')
TimeTree$tip.label <- sub('_', ' ', TimeTree$tip.label)


# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Prepare data -----------------------------------------------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
filter_small_spp <- function(x, minObs = 5) {
	# Filter species represented by too few observations
	x %>% 
		add_count(InoculatedSpecies) %>% 
		filter(n > minObs) %>% 
		add_count(SourceSpecies) %>% 
		filter(n > minObs) %>% 
		select(-n) %>% 
		as.data.frame()  # Don't return a tibble (does not work in MCMCglmm)
}

prepare_data <- function(data, lowerCol, upperCol) {
  lowerCol <- enquo(lowerCol)
  upperCol <- enquo(upperCol)
  
  FinalData <- data %>% 
    rename(Lower = !!lowerCol,
           Upper = !!upperCol) %>% 
    
    mutate(InoculatedKnownHost = ifelse(InoculatedKnownHost, 1, 0),
           InoculumTypeBrain = ifelse(InoculumType == 'Brain', 1, 0),   # Treating SalivaryGland and small number of Supernatant inocula as the same
    			 DonorIsBat = ifelse(SourceOrder == 'Chiroptera', 1, 0)) %>%   
    
    mutate(SupertreeDist_std = standardise_gelman( SupertreeDist ),
           logSupertreeDist_std = standardise_gelman( log1p(SupertreeDist) ),
           
           InoculumTypeBrain_cen = center_only( InoculumTypeBrain ),
    			 DonorIsBat_cen = center_only( DonorIsBat ),
    			 SourceBodyMass_std = standardise_gelman( SourceBodyMass ),
           InoculatedBodyMass_std = standardise_gelman( InoculatedBodyMass ),
           InoculatedKnownHost_cen = center_only( InoculatedKnownHost ),
           logDose_std = standardise_gelman( log10(Dose) ),
           InoculationPropDist_std = standardise_gelman( InoculationPropDist ),
           
           SourceBodyTemp_std = standardise_gelman( SourceBodyTemp ),
           InoculatedBodyTemp_std = standardise_gelman( InoculatedBodyTemp ),
           BodyTempDiff_std = standardise_gelman( BodyTempDiff ),
           
           InoculatedPhylo = InoculatedSpecies,
           SourcePhylo = SourceSpecies
    )
  
  FinalData
}

subset_phylo <- function(tree, keeptips) {
  keepIndex <- tree$tip.label %in% keeptips
  dropTips <- tree$tip.label[!keepIndex]
  
  drop.tip(tree, dropTips)
}



# Prepare data:
dfIncubation <- dfIncubation %>% 
 	filter(SourceSpecies != InoculatedSpecies) %>% 
	#	filter_small_spp() %>% 
  prepare_data(inc_nimLower, inc_nimUpper)

dfMorbidity <- dfMorbidity %>% 
	filter(SourceSpecies != InoculatedSpecies) %>% 
	#	filter_small_spp() %>% 
  prepare_data(morb_nimLower, morb_nimUpper)


# Save the exact data used in fitting to ensure plots are accurate:
dir.create('Output')
saveRDS(dfIncubation, 'Output/dfIncubation_includingDerivedVars.rds')
saveRDS(dfMorbidity, 'Output/dfMorbidity_includingDerivedVars.rds')


Ancestry <- inverseA(TimeTree)



# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Fits ---------------------------------------------------------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
feFormula <- log(cbind(Lower, Upper)) ~ logDose_std + InoculationPropDist_std + InoculumTypeBrain_cen + 
                  SupertreeDist_std + SupertreeDist_std:logDose_std +
                  
                  InoculatedKnownHost_cen + InoculatedKnownHost_cen:logDose_std +
									DonorIsBat_cen + DonorIsBat_cen:logDose_std +
                  InoculatedBodyMass_std + InoculatedBodyMass_std:logDose_std +
                  BodyTempDiff_std + BodyTempDiff_std:logDose_std

reFormula <- ~ ExperimentID + InoculatedPhylo + InoculatedSpecies + SourceSpecies + SourcePhylo


prior <- list(G = list(Gexp     = list(V = 1, nu = 1, alpha.mu = 0, alpha.V = 1000),  # Use parameter-expanded priors to aid mixing
											 GinPhylo = list(V = 1, nu = 1, alpha.mu = 0, alpha.V = 1000),
											 GinSp    = list(V = 1, nu = 1, alpha.mu = 0, alpha.V = 1000),
											 GsoPhylo = list(V = 1, nu = 1, alpha.mu = 0, alpha.V = 1000),
											 GsoSp    = list(V = 1, nu = 1, alpha.mu = 0, alpha.V = 1000)))


## Incubation period:
incFitFull <- MCMCglmm(fixed = feFormula, random = reFormula,
											 ginverse = list(InoculatedPhylo = Ancestry$Ainv, SourcePhylo = Ancestry$Ainv),
											 prior = prior, 
											 family = 'cengaussian',
											 data = dfIncubation,
											 nitt = ITERATIONS, thin = THIN, burnin = BURNIN, verbose = FALSE, pr = TRUE)

incFitFull <- list(Chain1 = incFitFull)
saveRDS(incFitFull, 'Output/incFitFull.rds')


## Morbidity period: remove source spp re's (pooling data, because there's only 4 source spp and low N)
reFormulaMorb <- update.formula(reFormula, ~ . - SourceSpecies - SourcePhylo)
priorMorb <- list(G = list(Gexp = prior$G$Gexp,
													 GinPhylo = prior$G$GinPhylo,
													 GinSpp = prior$G$GinSp))

morbFitFull <- MCMCglmm(fixed = feFormula, random = reFormulaMorb,
												ginverse = list(InoculatedPhylo = Ancestry$Ainv),
												prior = priorMorb, 
												family = 'cengaussian',
												data = dfMorbidity,
												nitt = ITERATIONS, thin = THIN, burnin = BURNIN, verbose = FALSE, pr = TRUE)

morbFitFull <- list(Chain1 = morbFitFull)
saveRDS(morbFitFull, 'Output/morbFitFull.rds')




## With separate body temperature variables:
#  - Need to remove inoculated body mass here
# 		- No effect in difference model
# 		- Correlated with inoculated body temp
# 		- Also no effect in a model containing source body temp but not inoc body temp
#  - Also need to remove IsBat
#     - Highly correlated with source body temp
#     - Use source body mass as a proxy
separatedFormula <- update.formula(feFormula,
																	 . ~ . - BodyTempDiff_std - BodyTempDiff_std:logDose_std -
																	 	InoculatedBodyMass_std - InoculatedBodyMass_std:logDose_std -
																	 	DonorIsBat_cen - DonorIsBat_cen:logDose_std +
																	 	SourceBodyMass_std + SourceBodyMass_std:logDose_std +
																	 	InoculatedBodyTemp_std + InoculatedBodyTemp_std:logDose_std + 
																	 	SourceBodyTemp_std + SourceBodyTemp_std:logDose_std)

incFitSeparate <- MCMCglmm(fixed = separatedFormula, random = reFormula,
													 ginverse = list(InoculatedPhylo = Ancestry$Ainv, SourcePhylo = Ancestry$Ainv),
													 prior = prior, 
													 family = 'cengaussian',
													 data = dfIncubation,
													 nitt = ITERATIONS, thin = THIN, burnin = BURNIN, verbose = FALSE, pr = TRUE)


incFitSeparate <- list(Chain1 = incFitSeparate)
saveRDS(incFitSeparate, 'Output/incFitSeparate.rds')
