# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---------Utility script---------
#   * Used by: MetaAnalysis_Main
#   * Requires: None
# -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# Loads and cleans meta-analysis data
#   * Loads meta-analysis data from postgresql database on iMac
#   * Loads sequence data and calculates genetic distance
#   * Loads host variables from Pantheria and AnAge database files
#   * Prepares/cleans data:
#       - Calculate genetic distance
#       - Calculate inoculation distance
#       - Remove problematic entries (TEMPORARY)
#       - Generate species codes (integers)
#       - Generate a variance/covariance matrix under a Brownian model from mammalian supertree (indexes match spp codes)
#   * Calculate additional survival times from incubation- and morbidity periods
#       - Also generates censoring codes compatible with survreg and JAGS
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

ExistingObjects <- ls()
library(DBI)
library(RPostgreSQL)
library(dplyr)
library(ape)
library(survival)
library(purrr)
library(purrrlyr)

source('Scripts/Utils/utils_survival.R')


## ---- Control the dataset produced -----------------------------------------------------------------------------
# These may also be set from outside, by a script sourcing this one
# Thus, don't overwrite values if they exist in the scope already, but provide defaults if they don't:

# Whether clinical-only diagnoses are still counted as 'RabiesConfirmed'
if (! exists("IncludeClinicalDiagnosis", mode = "logical")) {
  IncludeClinicalDiagnosis <- TRUE
  warning("Clinical diagnoses will be included by default, set IncludeClinicalDiagnosis to FALSE to avoid this")
}

# We will need to check whether these are skewing results
if (! exists("IncludePassagedViruses", mode = "logical")) {
  IncludePassagedViruses <- TRUE
  warning("Passaged viruses will be included by default, set IncludePassagedViruses to FALSE to avoid this")
}

# Remove within-species inoculations?
if (! exists("RemoveWithinSpp", mode = "logical")) {
  RemoveWithinSpp <- FALSE
  warning("Including all inoculations by default, set RemoveWithinSpp to TRUE to analyse only experiments containing (at least some) cross-species inoculations")
}



# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
## ---- Load raw data ------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=


AllData <- readRDS("Data/Internal/raw_data.rds")
InoculumData <- read.csv('Data/Internal/InoculumData.csv', stringsAsFactors = FALSE)  # Additional info on inocula


AllData <- AllData %>% 
  left_join(InoculumData, by = c('CitationKey', 'SourceSpecies', 'InoculatedSpecies', 'InoculumID'))


# Add a unique identifier, so observations can always be identified (even after splitting data
# into incubation, morbidity, etc.):
AllData$RowID <- 1:nrow(AllData)



# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Initial cleaning / subsetting -----------------------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
## Remove passaged animals (if needed)
AllData$PassageNumber <- ifelse(is.na(AllData$PassageNumber), 0, AllData$PassageNumber)

if (! IncludePassagedViruses) {
  cat("IncludePassagedViruses is FALSE:", sum(AllData$PassageNumber != 0), "rows will be removed\n")
  AllData <- AllData[AllData$PassageNumber == 0, ]
}


## Remove unconventional titration methods:
KnownTitrationRows <- AllData$TitrationMethod %in% c("MICLD50", "MLD50")  # assuming these are the same

UnknownMethodString <- unique(AllData$TitrationMethod[ !KnownTitrationRows ]) %>% paste(collapse="\n - ")
cat(sum(!KnownTitrationRows), " rows (", sum(!KnownTitrationRows)/nrow(AllData)*100, "%) ",
    "removed from AllData because dose unit is wrong - check these TitrationMethods:\n - ", 
    UnknownMethodString, "\n", sep = "")

AllData <- AllData[KnownTitrationRows, ]


## Remove control animals
NegativeControlRows <- is.na(AllData$SourceSpecies) | is.na(AllData$Dose)
AllData <- AllData[!NegativeControlRows, ]

NegativeControlRows2 <- AllData$Dose == 0  # need to remove NA's in Dose before doing this comparison
AllData <- AllData[!NegativeControlRows2, ]

cat(sum(NegativeControlRows) + sum(NegativeControlRows2), "negative control rows removed from AllData\n")


## Remvove animals that could not be assigned to a single experiment:
#  - In Parker1996, two experiments - with slight variations in virus titre - were 
#    performed, but for a handfull of animals it was not clear which experiment they
#    belong to
AllData <- AllData[!AllData$CitationKey == 'Parker1996', ]


## Fix known data-entry errors:
# - Vos2013 euthanised animals upon first appearance of symptoms, but this COD was not
#   recorded correctly recorded beyond animal 9 (from there onwards, the footnote for
#   euthanasia vs. rabies was swapped around). All timing data relating to these animals 
#   is correct)
vos_euth <- c('8', '9', '10', '11', '12', '14', '15', '16', '17')
vos_rab <- '13'
AllData$CauseOfDeath[AllData$CitationKey == 'Vos2013' & AllData$AnimalID %in% vos_euth] <- 'Euthanised'
AllData$CauseOfDeath[AllData$CitationKey == 'Vos2013' & AllData$AnimalID %in% vos_rab] <- 'Rabies'


## Remove survivors: These animals developed symptoms but recovered and survived long term
#   - Not clear whether this was rabies (though most tested positive after eventual euthanasia)
#   - Too few cases of this happenning to really probe this
WeirdSurvivorRows <- AllData$Comments %in% c('Survived, with sequelae', 'This individual survived', 
                                             'Sick for 15 days, but survived',
                                             'Goat survived for 7 weeks with symptoms (recovery starting from day 5) and was then euthanized ',
                                             'Antibody 0.08 at 14 days, 0.04 at 30 days. Recovered; Negative in both nervous system and salivary glands so have put clinical diagnosis')

AllData <- AllData[!WeirdSurvivorRows, ]
cat(sum(WeirdSurvivorRows), "unusual survivors with symptoms removed from AllData\n")



## Remove inoculations using viruses from non-maintenance hosts
##  - In reality this means the true virus reservoir could not be indentified
NonMaintenance <- is.na(AllData$SourceKnownHost)  # These are unresolved species
removedHosts <- AllData$SourceSpecies[NonMaintenance]
AllData <- AllData[!NonMaintenance, ]

NonMaintenance2 <- ! AllData$SourceKnownHost
removedHosts <- c(removedHosts, AllData$SourceSpecies[NonMaintenance2])
AllData <- AllData[!NonMaintenance2, ]

removedHostsString <- unique(removedHosts) %>% paste(collapse="\n - ")
cat("Removed ", sum(NonMaintenance) + sum(NonMaintenance2), 
    " rows listing non-maintenance hosts as SourceSpecies:\n - ",
    removedHostsString, "\n", sep = "")


## Remove animals not resolved to species-level (can't be matched to Pantheria data later on):
AllData <- AllData[!(AllData$InoculatedSpecies %in% c("Crocidura sp.")), ]  # Removes entire study: Tomori1980a


## Switch to species level (IGNORING SUB-SPECIES INFO)
#   This is needed because panTHERIA and AnAge only have species-level data
splitIn <- split_sub_species(AllData$InoculatedSpecies)
AllData$InculatedSubSpecies <- splitIn[, 2]
AllData$InoculatedSpecies <- splitIn[, 1]

splitSo <- split_sub_species(AllData$SourceSpecies)
AllData$SourceSubSpecies <- splitSo[, 2]
AllData$SourceSpecies <- splitSo[, 1]

rm(splitIn, splitSo)


## Remove within-spp inoculations (if needed)
if (RemoveWithinSpp) {
  RemoveIDs <- AllData %>% 
    group_by(ExperimentID) %>% 
    summarise(WithinOnly = all(SourceSpecies == InoculatedSpecies)) %>% 
    filter(WithinOnly) %>% 
    .$ExperimentID
  
  cat("RemoveWithinSpp is TRUE:", sum(AllData$ExperimentID %in% RemoveIDs), "records from experiments containing only within-species inoculations removed\n")
  AllData <- AllData[!AllData$ExperimentID %in% RemoveIDs, ]
}



## Check for other common issues
no_nas <- function(vec) {
  ! any( is.na(vec) )
}

stopifnot( no_nas(AllData$SourceOrder) )  # HostProperties table out of date - need to add info for missing species
stopifnot( no_nas(AllData$InoculatedOrder) )

# NA's in control / RE variables - these would be treated like a single group in model (not good)
stopifnot( no_nas(AllData$InoculumID))  # Not all control animals caught?
stopifnot( no_nas(AllData$InoculumType))

stopifnot( no_nas(AllData$ExperimentID))
stopifnot( no_nas(AllData$CitationKey))



## Unkown COD's:
# When the cause of death is not known, it is listed as "Unknown". NA's in this column in the DB
# are used to indicate LITERARALLY not applicable, i.e. no death before end of experiment:
AllData$CauseOfDeath <- ifelse(is.na(AllData$CauseOfDeath), "None", AllData$CauseOfDeath)


# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Load external host data -----------------------------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
dfPanTHERIA_raw <- read.delim('Data/External/PanTheria_DB/PanTHERIA_1-0_WR05_Aug2008.txt',
                              na.strings = '-999', stringsAsFactors = FALSE)

dfAnAge_raw <- read.delim("Data/External/AnAge/anage_data.txt")
dfAnAge_raw <- mutate(dfAnAge_raw, SppBinomial = paste(Genus, Species))


## Keep only useful columns (so problems are easier to spot)
#   NB: the two DB's contain some duplicated columns
#   - Prefer panTHERIA data (because AnAge data may come from reviews rather than primary data in some cases)
#   - At present none of the used variables occur in both (except metabolic rate, but there the units differ so we can't supplement missing data anyway)
dfPanTHERIA <- dfPanTHERIA_raw %>% select(Species = MSW05_Binomial,
                                          BasalMetRate = X18.1_BasalMetRate_mLO2hr,
                                          BasalMetRateMass = X5.2_BasalMetRateMass_g,
                                          BodyMass = X5.1_AdultBodyMass_g)

dfAnAge <- dfAnAge_raw %>% select(Species = SppBinomial,
                                  BodyTemp_Kelvin = Temperature..K.)


dfExternalData <- left_join(dfPanTHERIA, dfAnAge, by="Species")   # left join since we only want mammals - panTHERIA contains all extant and recently extinct mammal species, while AnAge contains other taxa too


## Add derived variables:
dfExternalData <- dfExternalData %>% 
  mutate(NormalisedBasalMetRate = BasalMetRate / BasalMetRateMass,
         BodyTemp_Celsius = BodyTemp_Kelvin - 273.15) %>%
  select(-BasalMetRate, -BasalMetRateMass, -BodyTemp_Kelvin)



# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Prepare and transform data for models ---------------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

## Numeric inoculation site based on (proportional) distance from brain (still same for all species)
#  Based on clustered zones described in InoculationRouteZones.pptx, with the most distant and shallow site
#  receiving a distance of 1, and intracerebral inoculations a distance of 0:
ZoneDistances <- data.frame("Head_IC" = 0,
                            "Head_IM" = 21/119,
                            "Head_SC" = 28/119,
                            
                            "Neck_IM" = 28/119,
                            "Neck_SC" = 35/119,
                            
                            "Torso_IM" = 56/119,
                            "Torso_SC" = 63/119,
                            
                            "Limbs_IM" = 112/119,
                            "Limbs_SC" = 119/119,
                            stringsAsFactors = F)
ZoneDistances <- tidyr::gather(ZoneDistances, key="InoculationZone_Depth", value="InoculationPropDist")


ZoneAssignments <- rbind(
  cbind("Head_IC", c("Intracerebral", "Optic nerve", "Intracranial", "Intracisternal", "IC", "IC intercranial")),
  cbind("Head_IM", c("IM-masseter", "IM-temporal", "IM occipital", "Submandibular", "Masseter", 
                     "IM masseter", "IM-Masseter", "IM-masseter(both.sides)", "IM-masseter(both)", "left & right masseter", 
                     "IM Right Masseter", "IM M.masseter", "IM Left Masseter", "IM-maseter", "masseter ", "muscle masseter ",
                     "temporal muscle (single)", "temporal muscle (both)", "deep intramuscular temporal ", 
                     "deep intramuscular masseter  ", "both masseter muscles")),
  cbind("Head_SC", c("Intranasal", "Intranasally", "Subcutaneous-lips")),
  cbind("Neck_IM", c("IM-neck", "IM-cervical", "IM-cervival(left)", "IM-deap-neck", "deep intramuscular cervical  ")),
  cbind("Neck_SC", c("Subcutaneous-neck", "subcutaneous(neck)")),
  cbind("Torso_IM", c("Deltoid", "IM-deltoid", "IM-Deltoid", "IM-pectoral", "IM-Dorsal")),
  cbind("Torso_SC", c("SC-deltoid")),
  cbind("Limbs_IM", c("IM-quadriceps femoris (right)", "IM-right hindleg", "IM-hind leg", "IM-leg", "IM right thigh",
                      "IM-LongDigitalExtensor(Pelvic)", "IM-AbductorDigitiQuinti(Pelvic)", "IM-gastrocnemius", 
                      "Right gastrocnemius muscle", "Gastrocnemicus", "IM-crural", "IM-HindFoot", "intramuscular thigh")),
  cbind("Limbs_SC", c("Peripheral-footpad", "SC-HindLimb")))

ZoneAssignments <- as.data.frame(ZoneAssignments, stringsAsFactors=F)
colnames(ZoneAssignments) <- c("InoculationZone_Depth", "InoculationRoute")

# These cannot be classified in this scheme:
UnclassifiedInoculationRoutes <- c("Multiple injections into nasal mucosa and turbinates", "IM", "oral", "Oral",
                                   "IM gastrocnemius, IM masseter")

AllData <- AllData[!(AllData$InoculationRoute %in% UnclassifiedInoculationRoutes), ]


## Also try inoculation site withouth depth (for exploratory analyses):
NoDepthDistances <- rbind(data.frame(Zone = "Intracranial", Sites = c("Head_IC"), Distance = 0),
                          data.frame(Zone = "Head", Sites = c("Head_IM", "Head_SC"), Distance = 1/14),
                          data.frame(Zone = "Neck", Sites = c("Neck_IM", "Neck_SC"), Distance = 1/7),
                          data.frame(Zone = "Torso", Sites = c("Torso_IM", "Torso_SC"), Distance = 3/7),
                          data.frame(Zone = "Limbs", Sites = c("Limbs_IM", "Limbs_SC"), Distance = 7/7))
colnames(NoDepthDistances) <- c("InoculationZone", "InoculationZone_Depth", "InoculationDist_NoDepth")


InoculationRoutes <- full_join(ZoneDistances, ZoneAssignments, by="InoculationZone_Depth") %>% 
  full_join(NoDepthDistances, by="InoculationZone_Depth")



# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
## ---- Join data sources ----------------------------------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
rowsbefore <- nrow(AllData)

AllData <- left_join(AllData, InoculationRoutes, by="InoculationRoute")

stopifnot(! any(is.na(AllData$InoculationPropDist))) # Check for unclassified inoculation routes
stopifnot(nrow(AllData) == rowsbefore)  # duplicated inoculation routes above will cause multiple matches and thus data duplication in join


stopifnot(all(AllData$SourceSpecies %in% dfExternalData$Species))  # Check if dfPantheria is up to date...
stopifnot(all(AllData$InoculatedSpecies %in% dfExternalData$Species))

AllData <- left_join(AllData, dfExternalData, by=c("SourceSpecies" = "Species")) %>% 
  rename(SourceBodyTemp = BodyTemp_Celsius,
         SourceMetRate = NormalisedBasalMetRate,
         SourceBodyMass = BodyMass) %>% 
  left_join(dfExternalData, by=c("InoculatedSpecies" = "Species")) %>% 
  rename(InoculatedBodyTemp = BodyTemp_Celsius,
         InoculatedMetRate = NormalisedBasalMetRate,
         InoculatedBodyMass = BodyMass)

stopifnot(nrow(AllData) == rowsbefore) # Join should not have had multiple matches

## Add derived variables:
AllData <- AllData %>% 
  mutate(BodyTempDiff = SourceBodyTemp - InoculatedBodyTemp,
         MetRateDiff = SourceMetRate - InoculatedMetRate,
         BodyMassDiff = SourceBodyMass - InoculatedBodyMass)


## Remove rows with missing temperature data:
rowsBefore <- nrow(AllData)

AllData <- AllData %>% 
  filter(! is.na(BodyTempDiff))

warning(paste('Removing', rowsBefore - nrow(AllData), 'records with missing body temperature data'))


#===========================================================================================================
# Fix taxonomy further to match SuperTree
#===========================================================================================================
AllData$SourceSpecies <- with(AllData, replace(SourceSpecies, SourceSpecies=='Felis catus', 'Felis silvestris'))
AllData$InoculatedSpecies <- with(AllData, replace(InoculatedSpecies, InoculatedSpecies=='Felis catus', 'Felis silvestris'))



# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Calculate additional times from incubation- and morbidity periods, and generate censoring codes ----
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
check_condition <- function(condition, marker="", message="Condition not met") {
  # Produce a warning if condition is FALSE
  if (! condition) {
    warning( paste(message, marker, sep=": ") )
  }
}

check_timing_consistency <- function(survived, survived_upper, data_row) {
  # Checks that timing data is internally consistent (where more than one type is filled in)
  marker <- data_row$CitationKey  # For error reporting
  incubation_lower <- data_row$IncubationPeriod_Lower 
  incubation_upper <- data_row$IncubationPeriod_Upper
  morbidity_lower <- data_row$MorbidityPeriod_Lower
  morbidity_upper <- data_row$MorbidityPeriod_Upper
  
  if (!(is.na(survived)) & is.na(survived_upper) ) { 
    # exact survival time:
    if (! is.na(incubation_lower))
      check_condition(survived >= incubation_lower, marker) # >= because morbidity period could be 0
      
    if (! is.na(incubation_upper))
      check_condition(survived >= incubation_upper, marker)
    
    if (! is.na(morbidity_lower))
      check_condition(survived > morbidity_lower, marker)
    
    if (! is.na(morbidity_upper))
      check_condition(survived > morbidity_upper, marker)
    
    if (!is.na(incubation_upper) & !is.na(morbidity_upper))
      check_condition(survived == (incubation_upper + morbidity_upper), marker) # upper and lower values have to be the same if DaysSurvived is known exactly...
  
  } else if (!is.na(survived) & !is.na(survived_upper)) { 
    #interval censored survival time:
    check_condition(survived < survived_upper, marker)
    
    if (! is.na(incubation_lower)) {
      check_condition(survived >= incubation_lower, marker)
      check_condition(survived_upper > incubation_lower, marker)
    }
    
    if (! is.na(incubation_upper))
      check_condition(survived_upper > incubation_upper, marker)
    
    if (! is.na(morbidity_lower)) {
      check_condition(survived > morbidity_lower, marker)
      check_condition(survived_upper > morbidity_lower, marker)
    }
    
    if (! is.na(morbidity_upper))
      check_condition(survived_upper > morbidity_upper, marker)
    
    if (!is.na(incubation_upper) & !is.na(morbidity_upper))
      check_condition(survived_upper == (incubation_upper + morbidity_upper), marker)
    if (!is.na(incubation_lower) & !is.na(morbidity_lower))
      check_condition(survived == (incubation_lower + morbidity_lower), marker)
  }
}


calculate_incubation_period <- function(rabiesConfirmed, causeOfDeath, experimentLength,
                                        daysSurvived, daysSurvived_lower, daysSurvived_upper,
                                        incubation_lower, incubation_upper, morbidity_lower, morbidity_upper) {
  # Consolidate all timing and event data for incubation periods for a given row in AllData:
  # Returns NULL if no data is available (in which case row should be removed)
  
  if (rabiesConfirmed) {
    if (! is.na(incubation_lower) & ! is.na(incubation_upper)) {
      if (incubation_lower == incubation_upper) {
        # Exact time
        return(list(
          inc_CensorCode = 0,
          inc_nimObservedTime = incubation_lower,
          inc_nimLower = incubation_lower,
          inc_nimUpper = incubation_lower
        ))
      } else {
        # Interval censored
        stopifnot(incubation_lower < incubation_upper)  # this would be a data-entry error
        
        return(list(
          inc_CensorCode = 1,
          inc_nimObservedTime = NA,
          inc_nimLower = incubation_lower,
          inc_nimUpper = incubation_upper
        ))
      }
    }
    
    if (! is.na(incubation_lower)) {
      return(list(
        inc_CensorCode = 2,
        inc_nimObservedTime = NA,
        inc_nimLower = incubation_lower,
        inc_nimUpper = Inf
      ))
    }
    
    if (! is.na(incubation_upper)) {
      stop("Left censoring found in incubation period data (not implemented)")
    }
    
    if (causeOfDeath == 'rabies') {
      # Can calculate timings using morbidity period:
      if (!is.na(daysSurvived) & !is.na(morbidity_lower) & !is.na(morbidity_upper)) {
        lower <- max(0, daysSurvived - morbidity_upper)
        upper <- daysSurvived - morbidity_lower
        
        if (lower == upper) {
          # Exact time
          return(list(
            inc_CensorCode = 0,
            inc_nimObservedTime = lower,
            inc_nimLower = lower,
            inc_nimUpper = lower
          ))
        } else {
          # Interval censored
          stopifnot(lower < upper)  # would indicate a data entry error
          
          return(list(
            inc_CensorCode = 1,
            inc_nimObservedTime = NA,
            inc_nimLower = lower,
            inc_nimUpper = upper
          ))
        }
      }
      
      if (!is.na(daysSurvived_lower) & !is.na(daysSurvived_upper) & !is.na(morbidity_lower) & !is.na(morbidity_upper)) {
        lower = max(0, daysSurvived_lower - morbidity_upper)
        upper = daysSurvived_upper - morbidity_lower
        
        if (lower == upper) {
          return(list(
            inc_CensorCode = 0,
            inc_nimObservedTime = lower,
            inc_nimLower = lower,
            inc_nimUpper = lower
          ))
        } else {
          stopifnot(lower < upper)
          
          return(list(
            inc_CensorCode = 1,
            inc_nimObservedTime = NA,
            inc_nimLower = lower,
            inc_nimUpper = upper
          ))
        }
      }
      
      if (!is.na(daysSurvived_lower) & !is.na(morbidity_upper)) {
        # Can still calculate lower limit at least
        lower <- daysSurvived_lower - morbidity_upper
        stopifnot(lower > 0)   # No usable data, but need to check why this happens
        
        return(list(
          inc_CensorCode = 2,
          inc_nimObservedTime = NA,
          inc_nimLower = lower,
          inc_nimUpper = Inf
        ))
      }
      
      # Can't calculate informative data, so remove row:
      return(NULL)
      
    } else {
      # No usable data, so remove row
      return(NULL)
    }
    
  } else {
    # No incubation period possible, so signal removal of row
    return(NULL)
  }
  
  stop("Reaching this point should not be possible if all tests above have two defined outcomes")
}


calculate_morbidity_period <- function(rabiesConfirmed, causeOfDeath, experimentLength,
                                       daysSurvived, daysSurvived_lower, daysSurvived_upper,
                                       incubation_lower, incubation_upper, morbidity_lower, morbidity_upper) {
  # Consolidate data for morbidity period:
  right_censored_morbidity <- function(daysSurvived, daysSurvived_lower, incubation_upper) {
    # this sub-chain of the flow diagram gets called twice
    if (!is.na(incubation_upper)) {
      if (!is.na(daysSurvived)) {
        lower <- daysSurvived - incubation_upper
        if (! lower > 0) return(NULL)  # Not informative - we already know all times are between 0 and Inf, so remove row
        
        return(list(
          morb_CensorCode = 2,
          morb_nimObservedTime = NA,
          morb_nimLower = lower,
          morb_nimUpper = Inf
        ))
      }
      
      if (!is.na(daysSurvived_lower)) {
        lower <- daysSurvived_lower - incubation_upper
        if (! lower > 0) return(NULL)  # Not informative
        
        return(list(
          morb_CensorCode = 2,
          morb_nimObservedTime = NA,
          morb_nimLower = lower,
          morb_nimUpper = Inf
        ))
      }
      
      # If we reach this point nothing can be calculated:
      return(NULL)
      
    } else {
      # Can't be calculated, so no usable data
      return(NULL)
    }
    
    stop("Reaching this point should not be possible if all tests have two outcomes")
  }
  
  # Main code
  if (rabiesConfirmed) {
    if (causeOfDeath == 'rabies') {
      
      if (!is.na(morbidity_lower) & !is.na(morbidity_upper)) {
        if (morbidity_lower == morbidity_upper) {
          return(list(
            morb_CensorCode = 0,
            morb_nimObservedTime = morbidity_lower,
            morb_nimLower = morbidity_lower,
            morb_nimUpper = morbidity_lower
          ))
        } else {
          stopifnot(morbidity_lower < morbidity_upper)
          
          return(list(
            morb_CensorCode = 1,
            morb_nimObservedTime = NA,
            morb_nimLower = morbidity_lower,
            morb_nimUpper = morbidity_upper
          ))
        }
      }
      
      if (!is.na(morbidity_lower)) {
        return(list(
          morb_CensorCode = 2,
          morb_nimObservedTime = NA,
          morb_nimLower = morbidity_lower,
          morb_nimUpper = Inf
        ))
      }
      
      if (!is.na(morbidity_upper))
        stop("Left censoring in morbidity data (not implemented)")
      
      if (!is.na(daysSurvived) & !is.na(incubation_lower) & !is.na(incubation_upper)) {
        lower <- max(0, daysSurvived - incubation_upper)
        upper <- daysSurvived - incubation_lower
        
        if (lower == upper) {
          return(list(
            morb_CensorCode = 0,
            morb_nimObservedTime = lower,
            morb_nimLower = lower,
            morb_nimUpper = lower
          ))
        } else {
          stopifnot(lower < upper)
          
          return(list(
            morb_CensorCode = 1,
            morb_nimObservedTime = NA,
            morb_nimLower = lower,
            morb_nimUpper = upper
          ))
        }
      }
      
      if (!is.na(daysSurvived_lower) & !is.na(daysSurvived_upper) & !is.na(incubation_lower) & !is.na(incubation_upper)) {
        lower <- max(0, daysSurvived_lower - incubation_upper)
        upper <- daysSurvived_upper - incubation_lower
        
        if (lower == upper) {
          return(list(
            morb_CensorCode = 0,
            morb_nimObservedTime = lower,
            morb_nimLower = lower,
            morb_nimUpper = lower
          ))
        } else {
          stopifnot(lower < upper)
          
          return(list(
            morb_CensorCode = 1,
            morb_nimObservedTime = NA,
            morb_nimLower = lower,
            morb_nimUpper = upper
          ))
        }
      }
      
      return(
        right_censored_morbidity(daysSurvived, daysSurvived_lower, incubation_upper)
      )
      
    } else {
      # COD not rabies, so morbidity is right censored:
      if (!is.na(morbidity_lower)) {
        return(list(
          morb_CensorCode = 2,
          morb_nimObservedTime = NA,
          morb_nimLower = morbidity_lower,
          morb_nimUpper = Inf
        ))
      }
      
      return(
        right_censored_morbidity(daysSurvived, daysSurvived_lower, incubation_upper)
      )
    }
  } else {
    # No morbidity period possible
    return(NULL)
  }
  
  stop("Reaching this point should not be possible if all tests have two defined outcomes")
}



## Apply these functions to generate a combined/full dataset first (for joint model):
removeRowsInc <- c()
removeRowsMorb <- c()

incColumns <- tibble()
morbColumns <- tibble()


for (rowi in 1:nrow(AllData)) {
  incRow <- calculate_incubation_period(
    rabiesConfirmed = AllData$RabiesConfirmed[rowi] | (IncludeClinicalDiagnosis & AllData$RabiesClinicalDiagnosis[rowi]),
    causeOfDeath = tolower(AllData$CauseOfDeath[rowi]),
    experimentLength = AllData$ExperimentLength[rowi],
    daysSurvived = AllData$DaysSurvived[rowi],
    daysSurvived_lower = AllData$DaysSurvived_Lower[rowi],
    daysSurvived_upper = AllData$DaysSurvived_Upper[rowi],
    incubation_lower = AllData$IncubationPeriod_Lower[rowi],
    incubation_upper = AllData$IncubationPeriod_Upper[rowi],
    morbidity_lower = AllData$MorbidityPeriod_Lower[rowi],
    morbidity_upper = AllData$MorbidityPeriod_Upper[rowi]
  )
  
  morbRow <- calculate_morbidity_period(
    rabiesConfirmed = AllData$RabiesConfirmed[rowi] | (IncludeClinicalDiagnosis & AllData$RabiesClinicalDiagnosis[rowi]),
    causeOfDeath = tolower(AllData$CauseOfDeath[rowi]),
    experimentLength = AllData$ExperimentLength[rowi],
    daysSurvived = AllData$DaysSurvived[rowi],
    daysSurvived_lower = AllData$DaysSurvived_Lower[rowi],
    daysSurvived_upper = AllData$DaysSurvived_Upper[rowi],
    incubation_lower = AllData$IncubationPeriod_Lower[rowi],
    incubation_upper = AllData$IncubationPeriod_Upper[rowi],
    morbidity_lower = AllData$MorbidityPeriod_Lower[rowi],
    morbidity_upper = AllData$MorbidityPeriod_Upper[rowi]
  )
  
  if (is.null(incRow)) {
    removeRowsInc <- c(removeRowsInc, rowi)
    incColumns <- rbind(incColumns,
                        data_frame(inc_CensorCode = NA, 
                                   inc_nimObservedTime = NA,
                                   inc_nimLower = NA, 
                                   inc_nimUpper = NA))
  } else {
    incColumns <- rbind(incColumns, incRow)
  }
  
  if (is.null(morbRow)) {
    removeRowsMorb <- c(removeRowsMorb, rowi)
    morbColumns <- rbind(morbColumns, 
                         data_frame(morb_CensorCode = NA, 
                                    morb_nimObservedTime = NA,
                                    morb_nimLower = NA, 
                                    morb_nimUpper = NA))
  } else {
    morbColumns <- rbind(morbColumns, morbRow)
  }
}


AllData <- cbind(AllData, incColumns, morbColumns)


# Some morbidity periods are exactly zero - this is impossible,
# so it must simply be "less than 1 day":
allzero <- AllData$morb_nimLower == 0 & AllData$morb_nimUpper == 0
AllData$morb_nimUpper[allzero] <- 1
AllData$morb_CensorCode[allzero] <- 1  # now interval censored
AllData$morb_nimObservedTime[allzero] <- NA


## Calculate alternative versions of these times (for plotting, etc.)
AllData <- AllData %>% 
  mutate(inc_survLower = nimtime_to_survtime(inc_CensorCode, inc_nimLower, inc_nimUpper)$Lower,
         inc_survUpper = nimtime_to_survtime(inc_CensorCode, inc_nimLower, inc_nimUpper)$Upper,
         inc_jagsLower = nimtime_to_jagstime(inc_CensorCode, inc_nimLower, inc_nimUpper)$Lower,
         inc_jagsUpper = nimtime_to_jagstime(inc_CensorCode, inc_nimLower, inc_nimUpper)$Upper,
         
         morb_survLower = nimtime_to_survtime(morb_CensorCode, morb_nimLower, morb_nimUpper)$Lower,
         morb_survUpper = nimtime_to_survtime(morb_CensorCode, morb_nimLower, morb_nimUpper)$Upper,
         morb_jagsLower = nimtime_to_jagstime(morb_CensorCode, morb_nimLower, morb_nimUpper)$Lower,
         morb_jagsUpper = nimtime_to_jagstime(morb_CensorCode, morb_nimLower, morb_nimUpper)$Upper)


## Subset data for specific models (to remove completely uninformative rows)
RemoveIndexInc <- 1:nrow(AllData) %in% removeRowsInc
RemoveIndexMorb <- 1:nrow(AllData) %in% removeRowsMorb

dfIncubation <- AllData[!RemoveIndexInc, ]
dfMorbidity <- AllData[!RemoveIndexMorb, ]

dfAllTimings <- AllData[! (RemoveIndexInc & RemoveIndexMorb), ]  # Rows with at least one of the timings known



# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
## ---- Phylogenetic data ----------------------------------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
AllSpp_Incubation <- unique(c(dfIncubation$SourceSpecies, dfIncubation$InoculatedSpecies))
AllSpp_Morbidity <- unique(c(dfMorbidity$SourceSpecies, dfMorbidity$InoculatedSpecies))

SppList <- c(AllSpp_Incubation, AllSpp_Morbidity) %>% 
	unique() %>% 
	paste(collapse = '\n')
cat(SppList, file = 'Data/External/timetree/SpeciesList.txt')  # Use this list to download the tree used in the next line from timetrees.org

SuperTree_All <- read.tree('Data/External/timetree/Species_timetree.nwk')
SuperTree_All$tip.label <- sub('_', ' ', SuperTree_All$tip.label)


# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
## ---- Calculate distance on supertree --------------------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
AllKnownSpp <- unique(c(AllSpp_Incubation, AllSpp_Morbidity))
UnknownTips <- SuperTree_All$tip.label[! SuperTree_All$tip.label %in% AllKnownSpp]
SuperTree_Known <- drop.tip(SuperTree_All, UnknownTips) # a smaller tree to speed up calculation

SuperTreeDistMat <- cophenetic.phylo(SuperTree_Known)

dfIncubation$SupertreeDist <- with(dfIncubation, get_dist(SourceSpecies, InoculatedSpecies, SuperTreeDistMat))
dfMorbidity$SupertreeDist <- with(dfMorbidity, get_dist(SourceSpecies, InoculatedSpecies, SuperTreeDistMat))
dfAllTimings$SupertreeDist <- with(dfAllTimings, get_dist(SourceSpecies, InoculatedSpecies, SuperTreeDistMat))



# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
## ---- Clean up, so only data remains in global environment -----------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
KeepObjects <- c(ExistingObjects, 'AllData', 'dfIncubation', 'dfMorbidity', 'dfAllTimings',
                 'dfPanTHERIA', 'dfAnAge', 'dfExternalData', 'SuperTree', 'SuperTree_Known')

CurrentObjects <- ls()
RemovableObjects <- CurrentObjects[! CurrentObjects %in% KeepObjects]
rm(list=RemovableObjects)
rm(CurrentObjects, RemovableObjects)

