## 
## Data summary stats for reporting in text:
## 
library(dplyr)
library(survival)
library(stringr)


# Read data actually used in model:
dfIncubation <- readRDS('Output/dfIncubation_includingDerivedVars.rds')
dfMorbidity <- readRDS('Output/dfMorbidity_includingDerivedVars.rds')

dfGlandTitre <- readRDS('Output/dfGlandTitre_includingDerivedVars.rds')


#============================================================================================================
### Total data size

dI <- dfIncubation %>% 
	select(RowID, SourceSpecies, InoculatedSpecies, SourceOrder, InoculatedOrder, ExperimentID, CitationKey)

dM <- dfMorbidity %>% 
	select(RowID, SourceSpecies, InoculatedSpecies, SourceOrder, InoculatedOrder, ExperimentID, CitationKey)

dG <- dfGlandTitre %>% 
	select(RowID, SourceSpecies, InoculatedSpecies, SourceOrder, InoculatedOrder, ExperimentID, CitationKey)

dAll <- dI %>% 
	full_join(dM) %>% 
	full_join(dG)


# Number of inoculations
nrow(dAll)

# Inoculated species
length(unique(dAll$InoculatedSpecies))

# Inoculated orders
unique(dAll$InoculatedOrder)

# Reservoir species
length(unique(dAll$SourceSpecies))

# Reservoir orders
unique(dAll$SourceOrder)

# Experiments
length(unique(dAll$ExperimentID))

# Publications
length(unique(dAll$CitationKey))

# Publication date range
dates <- unique(dAll$CitationKey) %>% 
	str_extract('[[:digit:]]+') %>% 
	as.numeric()

range(dates)

sum(dates < 1985)/length(dates)


#============================================================================================================
### Incubation period
# Median and range
kmfit <- survfit(Surv(inc_survLower, inc_survUpper, type = 'interval2') ~ 1, data = dfIncubation)
kmfit

kmtable <- summary(kmfit)
kmtable
ProportionSymptomatic <- 1 - kmtable$surv
min(kmtable$time[ProportionSymptomatic >= 0.95])  # Time by which 95% are showing symptoms
min(kmtable$time[ProportionSymptomatic >= 0.99])

# Inoculations
nrow(dfIncubation)

# Proportion CSTs
sum(dfIncubation$SourceSpecies != dfIncubation$InoculatedSpecies) / nrow(dfIncubation)

# Studies
length(unique(dfIncubation$CitationKey))
length(unique(dfIncubation$ExperimentID))

# Inocula
length(unique(dfIncubation$InoculumID))

# Orders / Species
length(unique(dfIncubation$SourceOrder))
length(unique(dfIncubation$SourceSpecies))

length(unique(dfIncubation$InoculatedOrder))
length(unique(dfIncubation$InoculatedSpecies))

# Combinations
length(unique(paste(dfIncubation$SourceSpecies, dfIncubation$InoculatedSpecies)))


#============================================================================================================
### Morbidity period
# Median and range
kmfit <- survfit(Surv(morb_survLower, morb_survUpper, type = 'interval2') ~ 1, data = dfMorbidity)
kmfit

plot(kmfit)

kmtable <- summary(kmfit)
kmtable
ProportionAlive <- 1 - kmtable$surv
min(kmtable$time[ProportionAlive > 0])
min(kmtable$time[ProportionAlive >= 0.92])
min(kmtable$time[ProportionAlive >= 0.95])  # Time by which 95% are showing symptoms

min(kmtable$time[ProportionAlive >= 1]) # Not entirely accurate - see plot
min(kmtable$time[ProportionAlive >= 0.9999])

# Inoculations
nrow(dfMorbidity)

# CSTs
sum(dfMorbidity$SourceSpecies != dfMorbidity$InoculatedSpecies) / nrow(dfMorbidity)

# Studies
length(unique(dfMorbidity$CitationKey))
length(unique(dfMorbidity$ExperimentID))

# Inocula
length(unique(dfMorbidity$InoculumID))

# Orders / Species
length(unique(dfMorbidity$SourceOrder))
length(unique(dfMorbidity$SourceSpecies))

length(unique(dfMorbidity$InoculatedOrder))
length(unique(dfMorbidity$InoculatedSpecies))

# Combinations
length(unique(paste(dfMorbidity$SourceSpecies, dfMorbidity$InoculatedSpecies)))




#============================================================================================================
### Virus in salivary glands

## GIVING DATA FOR EITHER DISTRIBUTION:
# Inoculations
nrow(dfGlandTitre)

# CSTs
sum(dfGlandTitre$SourceSpecies != dfGlandTitre$InoculatedSpecies) / nrow(dfGlandTitre)

# Studies
length(unique(dfGlandTitre$CitationKey))
length(unique(dfGlandTitre$ExperimentID))

# Inocula
length(unique(dfGlandTitre$InoculumID))

# Orders / Species
length(unique(dfGlandTitre$SourceOrder))
length(unique(dfGlandTitre$SourceSpecies))

length(unique(dfGlandTitre$InoculatedOrder))
length(unique(dfGlandTitre$InoculatedSpecies))

# Combinations
length(unique(paste(dfGlandTitre$SourceSpecies, dfGlandTitre$InoculatedSpecies)))

# --------------------------------------------------------------------------------------------------
## GIVING DATA FOR GLAND TITRE SPECIFICALLY:
dfGlandTitre <- dfGlandTitre %>% 
	filter(!is.na(GlandTitre))

# Inoculations
nrow(dfGlandTitre)

# CSTs
sum(dfGlandTitre$SourceSpecies != dfGlandTitre$InoculatedSpecies) / nrow(dfGlandTitre)

# Studies
length(unique(dfGlandTitre$CitationKey))
length(unique(dfGlandTitre$ExperimentID))

# Inocula
length(unique(dfGlandTitre$InoculumID))

# Orders / Species
length(unique(dfGlandTitre$SourceOrder))
length(unique(dfGlandTitre$SourceSpecies))

length(unique(dfGlandTitre$InoculatedOrder))
length(unique(dfGlandTitre$InoculatedSpecies))

# Combinations
length(unique(paste(dfGlandTitre$SourceSpecies, dfGlandTitre$InoculatedSpecies)))
