library(rprojroot)
library(dplyr)
library(tidyr)
library(ggplot2)
library(khroma)
library(cowplot)

setwd(find_rstudio_root_file())


PLOT_THEME <- theme_bw() +
	theme(strip.background = element_blank(),
				strip.text = element_text(size = 8),
				axis.title = element_text(size = 8),
				axis.text = element_text(size = 7),
				legend.title = element_text(size = 8),
				legend.text = element_text(size = 7),
				panel.grid = element_blank())


# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Read data used -----------------------------------------------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

# Data used
incData <- readRDS('Output/dfIncubation_includingDerivedVars.rds')

temperatureData <- read.delim('Data/External/AnAge/anage_data.txt', sep = '\t', quote = ) %>% 
	mutate(BodyTemp_Kelvin = Temperature..K.,
				 BodyTemp_Celsius = BodyTemp_Kelvin - 273.15) %>% 
	filter(Order %in% incData$InoculatedOrder)


# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Correlation -----------------------------------------------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
fit <- lm(BodyTempDiff ~ SupertreeDist, data = incData)

p <- ggplot(incData, aes(x = SupertreeDist, y = BodyTempDiff)) +
	geom_hline(yintercept = 0, linetype = 2) +
	geom_jitter(shape = 1, width = 2, height = 0.05) +
	geom_smooth(method = 'lm') +
	geom_rug(position = position_jitter(width = 1, height = 0.025)) +
	labs(x = 'Phylogenetic distance (My)', y = 'Body temperature difference (°C)') +
	PLOT_THEME

pdf(file = 'Plots/BodyTemp_Correlation.pdf', width = 3.75, height = 3)
p
dev.off()


print(summary(fit))


# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Body temp in major host groups -------------------------------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
used_spp_source <- temperatureData %>% 
	mutate(SppBinomial = paste(Genus, Species)) %>% 
	filter(SppBinomial %in% incData$SourceSpecies) %>% 
	mutate(Type = 'Reservoir sp.')

used_spp_inoc <- temperatureData %>% 
	mutate(SppBinomial = paste(Genus, Species)) %>% 
	filter(SppBinomial %in% incData$InoculatedSpecies) %>% 
	mutate(Type = 'Inoculated sp.')

used_spp_data <- bind_rows(used_spp_source, used_spp_inoc)


pmain <- ggplot(temperatureData, aes(x = BodyTemp_Celsius, fill = Order)) +
	geom_density(alpha = 0.6) +
	scale_fill_bright(guide = guide_legend(nrow = 2)) +
	labs(x = 'Body temperature (°C)', y = 'Density') +
	xlim(32, 41) +
	PLOT_THEME +
	theme(axis.text.x = element_blank(),
				axis.title.x = element_blank(),
				axis.ticks.x = element_blank(),
				plot.margin = margin(t = 5.5, r = 5.5, b = -2, l = 5.5, unit = 'pt'),
				legend.position = c(0.3,0.88),
				legend.key.height = unit(0.8, 'lines'),
				legend.background = element_blank(),
				legend.title = element_blank(),
				legend.spacing.x = unit(0.2, 'lines'))

pindicator <- ggplot(used_spp_data, aes(x = BodyTemp_Celsius, y = Type, colour = Order)) +
	geom_jitter(width = 0, height = 0.1, alpha = 0.7) +
	geom_hline(yintercept = 1.5, colour = 'grey80', size = 0.3) +
	scale_colour_bright(guide = FALSE) +
	labs(x = 'Body temperature (°C)') +
	xlim(32, 41) +
	PLOT_THEME +
	theme(axis.title.y = element_blank(),
				plot.margin = margin(t = -1, r = 5.5, b = 5.5, l = 5.5, unit = 'pt'))


p <- plot_grid(pmain, pindicator, 
							 nrow = 2, rel_heights = c(3, 1),
							 align = 'v', axis = 'lr')


pdf(file = 'Plots/BodyTemp_Distribution.pdf', width = 3.75, height = 3)
	p
dev.off()
