# An alternative display of raw data

library(dplyr)
library(ggplot2)
library(cowplot)
library(png)
library(grid)


PLOT_THEME <- theme_bw() +
	theme(strip.background = element_blank(),
				strip.text = element_text(size = 8),
				axis.title = element_text(size = 8),
				axis.text = element_text(size = 7),
				legend.title = element_text(size = 8),
				legend.text = element_text(size = 7))

## Data used
incData <- readRDS('Output/dfIncubation_includingDerivedVars.rds')
morbData <- readRDS('Output/dfMorbidity_includingDerivedVars.rds')
glandData <- readRDS('Output/dfGlandTitre_includingDerivedVars.rds')

# Merge:
incData <- incData %>% 
	select(SourceSpecies, InoculatedSpecies, 
				 SourceOrder, InoculatedOrder,
				 SourceFamily, InoculatedFamily,
				 RowID, InoculumID, ExperimentID)

morbData <- morbData %>% 
	select(SourceSpecies, InoculatedSpecies, 
				 SourceOrder, InoculatedOrder,
				 SourceFamily, InoculatedFamily,
				 RowID, InoculumID, ExperimentID)

glandData <- glandData %>%
	select(SourceSpecies, InoculatedSpecies, 
				 SourceOrder, InoculatedOrder,
				 SourceFamily, InoculatedFamily,
				 RowID, InoculumID, ExperimentID)


UsedData <- merge(incData, morbData, all = TRUE) %>% 
	merge(glandData, all = TRUE)


## Sort spp by order
spp_names <- c(UsedData$SourceSpecies, UsedData$InoculatedSpecies)
orders <- c(UsedData$SourceOrder, UsedData$InoculatedOrder)
spp_names <- spp_names[order(orders, spp_names)]
spp_names <- unique(spp_names)


# Counts
count_data <- UsedData %>% 
	group_by(SourceOrder, SourceSpecies, InoculatedOrder, InoculatedSpecies) %>% 
	summarise(N = n()) %>% 
	ungroup() %>% 
	arrange(SourceOrder, InoculatedOrder) %>% 
	mutate(SourceSpecies = factor(SourceSpecies, levels = spp_names),
				 InoculatedSpecies = factor(InoculatedSpecies, levels = spp_names),
				 Source_int = as.numeric(SourceSpecies),
				 Inoc_int = as.numeric(InoculatedSpecies))

source_order_data <- count_data %>% 
	group_by(SourceOrder) %>% 
	summarise(min_int = min(Source_int),
						max_int = max(Source_int))

inoc_order_data <- count_data %>% 
	group_by(InoculatedOrder) %>% 
	summarise(min_int = min(Inoc_int),
						max_int = max(Inoc_int))


## Plot
main_plot <- ggplot(count_data, aes(x = -Inf, xend = Inf, y = Source_int, yend = Inoc_int, size = N, colour = InoculatedOrder)) +
	geom_segment() +
	geom_segment(aes(x = -Inf, xend = -Inf, y = min_int - 0.3, yend = max_int + 0.3, colour = SourceOrder),
							 size = 5, data = source_order_data) +
	geom_segment(aes(x = Inf, xend = Inf, y = min_int - 0.3, yend = max_int + 0.3, colour = InoculatedOrder),
							 size = 5, data = inoc_order_data) +
	scale_size(range = c(0.2, 2)) +
	scale_y_continuous(name = 'Virus reservoir',
										 breaks = as.numeric(unique(count_data$SourceSpecies)),
										 minor_breaks = 1:max(as.numeric(c(count_data$SourceSpecies, count_data$InoculatedSpecies))),
										 labels = unique(count_data$SourceSpecies),
										 sec.axis = sec_axis(trans = ~ ., name = 'Inoculated species',
										 										breaks = as.numeric(unique(count_data$InoculatedSpecies)), 
										 										labels = unique(count_data$InoculatedSpecies))) +
	labs(colour = 'Order', size = 'Number of\ninoculations') +
	xlim(c(0, 1)) +
	PLOT_THEME +
	theme(axis.text.x = element_blank(),
				axis.ticks.x = element_blank(),
				axis.title = element_blank(),
				panel.grid.major.x = element_blank(),
				panel.grid.minor.x = element_blank(),
				panel.grid.minor.y = element_line(size = 0.5),
				axis.text = element_text(size = 5, face = 'italic'),
				legend.key.height = unit(0.8, 'lines'),
				legend.margin = margin(t = 2, r = 5.5, b = 0, l = 5.5),
				plot.margin = margin(t = 20, r = 5.5, b = 20, l = 5.5))


## Add reservoir silhouttes
artiodactyl_img <- 'Data/External/Silhouttes/Bovid.svg'
carnivore_img <- 'Data/External/Silhouttes/Fox_base.svg'
bat_img <- 'Data/External/Silhouttes/Bat.svg'
rodent_img <- 'Data/External/Silhouttes/Rodent.svg'


# A white box to cover existing legend:
white_box <- rectGrob(
	gp = gpar(fill = "white", col = "white"),
	hjust = 0, vjust = 0
)


## Add text and reservoir annotations and save
box1_y <- 0.7725
outline1_y <- box1_y + 0.0686
spacing <- 0.085

p <- ggdraw(main_plot) +
	draw_grob(white_box,        x = 0.715, y = box1_y,                  width = 0.1, height = 0.13, clip = TRUE) +
	draw_image(artiodactyl_img, x = 0.74, y = outline1_y,              width = 0.1, height = 0.06) +
	
	draw_grob(white_box,        x = 0.715, y = box1_y - spacing,        width = 0.1, height = 0.13, clip = TRUE) +
	draw_image(carnivore_img,   x = 0.74, y = outline1_y - spacing,    width = 0.1, height = 0.06) +
	
	draw_grob(white_box,        x = 0.715, y = box1_y - 2*spacing,      width = 0.1, height = 0.13, clip = TRUE) +
	draw_image(bat_img,         x = 0.74, y = outline1_y - 2*spacing,  width = 0.1, height = 0.06) +
	
	draw_grob(white_box,        x = 0.715, y = box1_y - 3*spacing,      width = 0.1, height = 0.13, clip = TRUE) +
	draw_image(rodent_img,      x = 0.74, y = outline1_y - 3*spacing - 0.008,  width = 0.1, height = 0.06) +
	
	# Text labels
	draw_text(c('Virus reservoir', 'Host'), 
						x = c(0.09, 0.583), y = 0.96, size = 7) +
	draw_label(expression(phantom(0) %->% '   Inoculation   ' %->% ''),
						 x = 0.355, y = 0.96, size = 7) +
	draw_label(expression('[virus effects]'), x = 0.083, y = 0.04, size = 7) +
	draw_label(expression("[" %<-% "host-virus interactions" %->% "]"), x = 0.355, y = 0.04, size = 7) +
	draw_label(expression('[host effects]'), x = 0.627, y = 0.04, size = 7)


dir.create('Plots')
ggsave2(filename = 'Plots/SourceInoculatedSpeciesCombinations_Network.pdf', p, width = 4, height = 2.3, dpi = 600)

