
library(ggplot2)
library(ggstance)
library(ggmcmc)
library(dplyr)
library(tidyr)
library(broom)
library(coda)
library(MCMCglmm)

source('Scripts/Utils/utils_mcmcglmm.R')


# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Constants ----------------------------------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
LOG_DOSE_EXPR = "plain(log)[10]~bgroup('(',plain(Dose),')')"

FIXED_EFFECT_NAMES <- tribble(~Parameter,                           ~Label,                                      ~Group,
															'SupertreeDist_std',                   'Phylogenetic distance',                     'Host-virus\ninteraction',
															'logDose_std:SupertreeDist_std',       'Phylogenetic distance x Dose',              'Host-virus\ninteraction',
															
															'BodyTempDiff_std',                    'Body temperature difference',               'Host-virus\ninteraction',
															'logDose_std:BodyTempDiff_std',        'Body temperature difference x Dose',        'Host-virus\ninteraction',
															
															'SourceBodyMass_std',                  'Reservoir body mass',                       'Virus',
															'logDose_std:SourceBodyMass_std',      'Reservoir body mass x Dose',                'Virus',
															
															'DonorIsBat_cen',                      'Bat reservoir',                             'Virus',
															'logDose_std:DonorIsBat_cen',          'Bat reservoir x Dose',                      'Virus',
															
															'InoculatedBodyMass_std',              'Inoculated species body mass',              'Host',
															'logDose_std:InoculatedBodyMass_std',  'Inoculated species body mass x Dose',       'Host',
															
															'InoculatedKnownHost_cen',             'Known reservoir inoculated',                 'Host',
															'logDose_std:InoculatedKnownHost_cen', 'Known reservoir inoculated x Dose',           'Host',
															
															'logDose_std',                         LOG_DOSE_EXPR,                               'Experiment',
															'InoculationPropDist_std',             'Inoculation site distance',                 'Experiment',
															'InoculumTypeBrain_cen',               'Brain-material inoculated',                 'Experiment',
															
															'SourceBodyTemp_std',                  'Reservoir body temperature',                'Body\ntemperature',
															'logDose_std:SourceBodyTemp_std',      'Reservoir body temperature x Dose',         'Body\ntemperature',
															'InoculatedBodyTemp_std',              'Inoculated species body temperature',       'Body\ntemperature',
															'logDose_std:InoculatedBodyTemp_std',  'Inoculated species body temperature x Dose','Body\ntemperature'
															)
FIXED_EFFECT_NAMES <- FIXED_EFFECT_NAMES %>% 
	mutate(Label = gsub(' ', '~', Label),
				 Label = if_else(Parameter != 'logDose_std', paste0('plain(', Label, ')'), Label)) %>% 
	mutate(Label = factor(Label, levels = rev(.$Label)),
				 Group = factor(Group, levels = unique(.$Group)))



PLOT_THEME <- theme_bw() +
	theme(strip.background = element_blank(),
				strip.text = element_text(size = 8),
				axis.title = element_text(size = 8),
				axis.text = element_text(size = 7),
				legend.title = element_text(size = 8),
				legend.text = element_text(size = 7), 
				panel.grid = element_blank())

# Palletes: use different colour schemes for incubation and morbidity results:
INCUBATION_COLOUR <- '#8AC2DE'  # Blue
INCUBATION_DARK <- '#034e7b'

MORBIDITY_COLOUR <- '#A8DDAB'   # Green
MORBIDITY_DARK <- '#027207'

SINGLE_GREY <- 'grey70'

# Pure black looks too extreme:
DEFAULT_BLACK <- '#363636'


# Save plots:
save_pdf <- function(p, filename, width, height) {
	pdf(file = filename, width = width, height = height)
	  print(p)
	dev.off()
}


# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Load fitted models -------------------------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

IncFits <- readRDS('Output/incFitFull.rds')
IncFitSeparate <- readRDS('Output/incFitSeparate.rds')

MorbFits <- readRDS('Output/morbFitFull.rds')

dfIncubation <- readRDS('Output/dfIncubation_includingDerivedVars.rds')
dfMorbidity <- readRDS('Output/dfMorbidity_includingDerivedVars.rds')


dir.create('Plots')


# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Fixed effect coefficients ------------------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
get_coefs <- function(chains, name, fixed_effect_names = FIXED_EFFECT_NAMES) {
	Coefs <- lapply(chains, function(fit) fit$Sol[, colnames(fit$X)]) %>%   # Get fixed effect posterior
		as.mcmc.list()
	
	CoefSummary <- summarise_posterior(Coefs, chainsSeparate = FALSE) %>% 
		filter(Parameter != '(Intercept)') %>% 
		mutate(Significant = !(lower < 0 & upper > 0)) %>% 
		left_join(fixed_effect_names, by = 'Parameter') %>% 
		mutate(Name = name)
	
	CoefDistribution <- ggs(Coefs) %>% 
		filter(Parameter != '(Intercept)') %>% 
		left_join(CoefSummary, by = 'Parameter') %>% 
		mutate(Name = name)
	
	# Return:
	list(Summary = CoefSummary,
			 Distribution = CoefDistribution)
}

plot_coefs <- function(chains1, singleColour, chains2 = NULL, facetNames = NULL, 
											 xlimits = c(-1.5, 1.5), fixed_effect_names = FIXED_EFFECT_NAMES) {
	if (is.null(chains2)) 
		facetNames <- 'A'  # Name not used when plotting only one set of coefficients
	
	Coefs <- get_coefs(chains1, facetNames[1], fixed_effect_names = fixed_effect_names)
	
	if (! is.null(chains2)) {
		# Plot another set of coefs alongside:
		temp <- get_coefs(chains2, facetNames[2])
		Coefs$Summary <- bind_rows(Coefs$Summary, temp$Summary)
		Coefs$Distribution <- bind_rows(Coefs$Distribution, temp$Distribution)
		
		Coefs$Summary$Name <- factor(Coefs$Summary$Name, levels = facetNames)
		Coefs$Distribution$Name <- factor(Coefs$Distribution$Name, levels = facetNames)
	}
	
	
	p <- ggplot(Coefs$Summary, aes(y = Label, x = median, colour = Significant, fill = Significant)) +
		geom_vline(xintercept = 0, linetype = 2) +
		geom_violinh(aes(x = value), colour = NA, data = Coefs$Distribution) +
		geom_linerangeh(aes(xmin = lower, xmax = upper)) +
		geom_point() +
		scale_color_manual(values = c('TRUE' = DEFAULT_BLACK, 'FALSE' = 'grey30'), guide = F) +
		scale_fill_manual(values = c('TRUE' = singleColour, 'FALSE' = SINGLE_GREY), guide = F) +
		scale_y_discrete(name = NULL, label = function(x) parse(text = x)) +
		scale_x_continuous(name = 'Effect size', breaks = seq(min(xlimits), max(xlimits), by = 1)) +
		coord_cartesian(xlim = xlimits) +
		PLOT_THEME
	
	if (! is.null(chains2)) {
		p <- p + facet_grid(Group ~ Name, scales = 'free', space = 'free', shrink = T)
	} else {
		p <- p + facet_grid(Group ~ ., scales = 'free_y', space = 'free', shrink = T)
	}
	
	p
}	

p1 <- plot_coefs(IncFits, INCUBATION_COLOUR, 
								 MorbFits, c('Incubation period', 'Clinical period'),
								 xlimits = c(-3, 3.5))

save_pdf(p1, 'Plots/Fig2_Coefs_Combined.pdf', width = 6, height = 3.4)



# Seperate effects:
p3 <- plot_coefs(IncFitSeparate, INCUBATION_COLOUR, xlimits = c(-3, 2)) +
	theme(strip.text = element_text(size = 6))

save_pdf(p3, 'Plots/Supplement1_Coefs_IncubationSeperate.pdf', width = 5, height = 3.5)




# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Predictions: BodyTempDiff x Dose -----------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# -> Plotting predictions for first chain
# -> Limiting dose to area with 90% of observations to avoid predicting in areas with little data
doseLims <- quantile(log10(dfIncubation$Dose), c(0.05, 0.95))  # 90% quantile 
tempDiffLims <- quantile(dfIncubation$BodyTempDiff, c(0.05, 0.95))


preds <- predict_marginal_effect(IncFits[[1]], 'BodyTempDiff_std', 'logDose_std', resolution = 0.05) %>% 
	mutate(BodyTempDiff = rev_standardise_gelman(BodyTempDiff_std, dfIncubation$BodyTempDiff),
				 logDose = rev_standardise_gelman(logDose_std, log10(dfIncubation$Dose)))

# Ensure colour scale matches displayed data:
colourLims <- preds %>% 
	filter(logDose >= doseLims[1] & logDose <= doseLims[2]) %>% 
	filter(BodyTempDiff >= tempDiffLims[1]-0.19 & BodyTempDiff <= tempDiffLims[2]) %>% 
	.$fit %>% 
	exp() %>% 
	range()

p1 <- ggplot(preds, aes(x = BodyTempDiff, y = logDose)) +
	geom_raster(aes(fill = exp(fit))) +
	geom_rug(aes(y = log10(Dose)), alpha = 0.6, data = dfIncubation,
					 position = position_jitter(width = 0.05, height = 0.05)) +
	geom_vline(xintercept = 0, linetype = 2) +
	
	scale_fill_viridis_c(direction = -1, limits = colourLims) +
	
	labs(x = 'Body temperature difference (°C)', y = parse(text = LOG_DOSE_EXPR), 
			 fill = 'Incubation period (days)') +
	coord_cartesian(xlim = tempDiffLims, ylim = doseLims, expand = F) +
	guides(fill = guide_colourbar(label.position = 'top')) +
	PLOT_THEME +
	theme(legend.position = 'top',
				legend.key.height = unit(0.6, 'lines'),
				legend.title = element_text(vjust = 0.13),
				legend.margin = margin(b = 0.4, unit = 'lines'))


save_pdf(p1, 'Plots/Fig2_Inc_BodyTempXDose_Grid.pdf', width = 3, height = 2) #3 x 2



# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Predictions: Separate body temp vars x Dose -----------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# These plots need to comparable side-by-side, so all limits should match (including colour scale):
TempLims <- range(c(dfIncubation$SourceBodyTemp, dfIncubation$InoculatedBodyTemp))

predsA <- predict_marginal_effect(IncFitSeparate[[1]], 'SourceBodyTemp_std', 'logDose_std', resolution = 0.05) %>% 
	mutate(SourceBodyTemp = rev_standardise_gelman(SourceBodyTemp_std, dfIncubation$SourceBodyTemp),
				 logDose = rev_standardise_gelman(logDose_std, log10(dfIncubation$Dose)))

predsB <- predict_marginal_effect(IncFitSeparate[[1]], 'InoculatedBodyTemp_std', 'logDose_std', resolution = 0.05) %>% 
	mutate(InoculatedBodyTemp = rev_standardise_gelman(InoculatedBodyTemp_std, dfIncubation$InoculatedBodyTemp),
				 logDose = rev_standardise_gelman(logDose_std, log10(dfIncubation$Dose)))


IncLims <- bind_rows(predsA, predsB) %>% 
	filter(logDose >= doseLims[1] & logDose <= doseLims[2]) %>% 
	.$fit %>% 
	exp() %>% 
	range()


p1 <- ggplot(predsA, aes(x = SourceBodyTemp, y = logDose)) +
	geom_raster(aes(fill = exp(fit))) +
	geom_rug(aes(y = log10(Dose)), alpha = 0.6, data = dfIncubation,
					 position = position_jitter(width = 0.05, height = 0.05)) +
	
	scale_fill_viridis_c(direction = -1, limits = IncLims) +
	
	labs(x = 'Reservoir body temperature (°C)', y = parse(text = LOG_DOSE_EXPR), 
			 fill = 'Incubation period (days)') +
	coord_cartesian(ylim = doseLims, xlim = TempLims, expand = FALSE) +
	guides(fill = guide_colourbar(label.position = 'top')) +
	PLOT_THEME +
	theme(legend.position = 'top',
				legend.key.height = unit(0.6, 'lines'),
				legend.title = element_text(vjust = 0.13),
				legend.margin = margin(b = -0.3, unit = 'lines'))


p2 <- ggplot(predsB, aes(x = InoculatedBodyTemp, y = logDose)) +
	geom_raster(aes(fill = exp(fit))) +
	geom_rug(aes(y = log10(Dose)), alpha = 0.6, data = dfIncubation,
					 position = position_jitter(width = 0.05, height = 0.05)) +

	scale_fill_viridis_c(direction = -1, limits = IncLims) +
	
	labs(x = 'Inoculated species body temperature (°C)', y = parse(text = LOG_DOSE_EXPR), 
			 fill = 'Incubation period (days)') +
	coord_cartesian(ylim = doseLims, xlim = TempLims, expand = FALSE) +
	guides(fill = guide_colourbar(label.position = 'top')) +
	PLOT_THEME +
	theme(legend.position = 'top',
				legend.key.height = unit(0.6, 'lines'),
				legend.title = element_text(vjust = 0.13),
				legend.margin = margin(b = -0.3, unit = 'lines'))


save_pdf(p1, 'Plots/Supplement_Separate_SourceBodyTemp.pdf', width = 3, height = 2)
save_pdf(p2, 'Plots/Supplement_Separate_InocBodyTemp.pdf', width = 3, height = 2)


# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Predictions: IsBat x Dose ------------------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# Show predicted distributions at low, medium and high doses...

probs <- c(0.125, 0.25, 0.5)
DoseQs <- quantile(log10(dfIncubation$Dose), probs = probs) %>% 
	standardise_gelman(Mean = mean(log10(dfIncubation$Dose)), SD = sd(log10(dfIncubation$Dose)))

names(probs) <- as.character(DoseQs)


predsMedian <- predict_marginal_effect(IncFits[[1]], 'DonorIsBat_cen', 'logDose_std', 
																			 var1Values = unique(dfIncubation$DonorIsBat_cen),
																			 var2Values = DoseQs)

predDistribution <- predict_marginal_distribution(IncFits[[1]], 'DonorIsBat_cen', 'logDose_std', 
																									var1Values = unique(dfIncubation$DonorIsBat_cen),
																									var2Values = DoseQs)
saveRDS(predDistribution, 'Output/predDistribution_BatXDose.rds')


predsMedian <- predsMedian %>% 
	mutate(DonorIsBat = rev_center_only(DonorIsBat_cen, dfIncubation$DonorIsBat),
				 logDose = rev_standardise_gelman(logDose_std, log10(dfIncubation$Dose)),
				 Quartile = probs[as.character(logDose_std)],
				 DoseLabel = sprintf('Q[%f]==%.2f', Quartile, logDose)) %>% 
	mutate(ReservoirOrder = if_else(DonorIsBat == 0, 'Carnivora', 'Chiroptera'))

predDistribution <- predDistribution %>% 
	mutate(DonorIsBat = rev_center_only(DonorIsBat_cen, dfIncubation$DonorIsBat),
				 logDose = rev_standardise_gelman(logDose_std, log10(dfIncubation$Dose)),
				 Quartile = probs[as.character(logDose_std)],
				 DoseLabel = sprintf('Q[%f]==%.2f', Quartile, logDose)) %>% 
	mutate(ReservoirOrder = if_else(DonorIsBat == 0, 'Carnivora', 'Chiroptera'))



p1 <- ggplot(predsMedian, aes(x = ReservoirOrder, y = fit)) +
	geom_violin(data = predDistribution, fill = INCUBATION_COLOUR, colour = NA) +
	
	geom_point(colour = INCUBATION_DARK, size = 2) +
	geom_linerange(aes(ymin = lwr, ymax = upr), colour = INCUBATION_DARK) +
	geom_line(aes(group = DoseLabel), linetype = '12', colour = DEFAULT_BLACK) +
	
	
	facet_grid(. ~ DoseLabel, labeller = label_parsed) +
	labs(x = 'Reservoir taxonomic order', y = expression(log(plain(Incubation~period)))) +
	coord_cartesian(ylim = c(0, 6)) +
	PLOT_THEME +
	theme(axis.text.x = element_text(angle = 20, vjust = 0.8),
				axis.ticks.length = unit(0.2, 'lines'))


save_pdf(p1, 'Plots/Fig2_Inc_IsBatXDose.pdf', width = 3, height = 2)


# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Predictions: KnownHost x Dose (clinical period) --------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
probs <- c(0.125, 0.25, 0.5)
DoseQs <- quantile(log10(dfMorbidity$Dose), probs = probs) %>% 
	standardise_gelman(Mean = mean(log10(dfMorbidity$Dose)), SD = sd(log10(dfMorbidity$Dose)))

names(probs) <- as.character(DoseQs)

predsMedian <- predict_marginal_effect(MorbFits[[1]], 'InoculatedKnownHost_cen', 'logDose_std', 
																			 var1Values = unique(dfMorbidity$InoculatedKnownHost_cen),
																			 var2Values = DoseQs)

predDistribution <- predict_marginal_distribution(MorbFits[[1]], 'InoculatedKnownHost_cen', 'logDose_std', 
																									var1Values = unique(dfMorbidity$InoculatedKnownHost_cen),
																									var2Values = DoseQs)
saveRDS(predDistribution, 'Output/predDistribution_KnownHostXDose.rds')


predsMedian <- predsMedian %>% 
	mutate(InoculatedKnownHost = rev_center_only(InoculatedKnownHost_cen, dfMorbidity$InoculatedKnownHost),
				 logDose = rev_standardise_gelman(logDose_std, log10(dfMorbidity$Dose)),
				 Quartile = probs[as.character(logDose_std)],
				 DoseLabel = sprintf('Q[%f]==%.2f', Quartile, logDose)) %>% 
	mutate(InoculatedKnownHost = if_else(InoculatedKnownHost == 1, 'True', 'False'))

predDistribution <- predDistribution %>% 
	mutate(InoculatedKnownHost = rev_center_only(InoculatedKnownHost_cen, dfMorbidity$InoculatedKnownHost),
				 logDose = rev_standardise_gelman(logDose_std, log10(dfMorbidity$Dose)),
				 Quartile = probs[as.character(logDose_std)],
				 DoseLabel = sprintf('Q[%f]==%.2f', Quartile, logDose)) %>% 
	mutate(InoculatedKnownHost = if_else(InoculatedKnownHost == 1, 'True', 'False'))



p1 <- ggplot(predsMedian, aes(x = InoculatedKnownHost, y = fit)) +
	geom_violin(data = predDistribution, fill = MORBIDITY_COLOUR, colour = NA) +
	
	geom_point(colour = MORBIDITY_DARK, size = 2) +
	geom_linerange(aes(ymin = lwr, ymax = upr), colour = MORBIDITY_DARK) +
	geom_line(aes(group = DoseLabel), linetype = '12', colour = DEFAULT_BLACK) +
	
	
	facet_grid(. ~ DoseLabel, labeller = label_parsed) +
	labs(x = 'Known reservoir inoculated', y = expression(log(plain(Clinical~period)))) +
	coord_cartesian(ylim = c(-2, 4)) +
	PLOT_THEME


save_pdf(p1, 'Plots/Fig2_Morb_KnownHostXDose.pdf', width = 3, height = 2)




# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Predictions: Phylo dist x Dose (morbidity) -------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# -> Plotting predictions for first chain
# -> Limiting both axes to area with ~90% of observations to avoid making predictions in areas with
#    very little data (e.g. extreme doses)

doseLims <- quantile(log10(dfMorbidity$Dose), c(0.05, 0.95))  # 90% quantile 
phylodistLims <- quantile(dfMorbidity$SupertreeDist, c(0.05, 0.95))

preds <- predict_marginal_effect(MorbFits[[1]], 'SupertreeDist_std', 'logDose_std', resolution = 0.05) %>% 
	mutate(SupertreeDist = rev_standardise_gelman(SupertreeDist_std, dfMorbidity$SupertreeDist),
				 logDose = rev_standardise_gelman(logDose_std, log10(dfMorbidity$Dose)))

colourLims <- preds %>% 
	filter(logDose >= doseLims[1] & logDose <= doseLims[2]+0.061) %>% 
	filter(SupertreeDist >= phylodistLims[1]-0.05 & SupertreeDist <= phylodistLims[2]) %>% 
	.$fit %>% 
	exp() %>% 
	range()


phylodistLims[2] <- min(phylodistLims[2], max(preds$SupertreeDist))


p1 <- ggplot(preds, aes(x = SupertreeDist, y = logDose)) +
	geom_raster(aes(fill = exp(fit))) +
	geom_rug(aes(y = log10(Dose)), alpha = 0.6, data = dfMorbidity,
					 position = position_jitter(width = 0.05, height = 0.05)) +
	
	scale_fill_viridis_c(direction = -1, limits = colourLims) +
	
	labs(x = 'Phylogenetic distance (My)', y = parse(text = LOG_DOSE_EXPR), 
			 fill = 'Clinical period (days)') +
	coord_cartesian(xlim = phylodistLims, ylim = doseLims, expand = FALSE) +
	guides(fill = guide_colourbar(label.position = 'top')) +
	PLOT_THEME +
	theme(legend.position = 'top',
				legend.key.height = unit(0.6, 'lines'),
				legend.title = element_text(vjust = 0.13),
				legend.margin = margin(b = -0.3, unit = 'lines'))
p1

save_pdf(p1, 'Plots/Fig2_Morb_PhyloDistXDose.pdf', width = 3, height = 2)




# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Residuals at different body temperatures ---------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
residsInc <- get_leftbound_residuals(chains = IncFits, obsLower = log(dfIncubation$Lower), obsUpper = log(dfIncubation$Upper))

incDat <- dfIncubation %>% 
	mutate(ObsID = row_number(),
				 ExpectedUpper = log(Lower) - log(Upper),
				 BodyTempDiff_jit = jitter(BodyTempDiff, amount = 0.1),
				 Dose = cut_interval(log10(Dose), 3)) 

residsInc <- residsInc %>% 
	filter(Chain == 1) %>% 
	left_join(incDat, by = 'ObsID') %>% 
	mutate(Censoring = if_else(Lower == Upper, 'Exact', 
														 if_else(is.infinite(Upper), 'Right', 'Interval')))


p1 <- ggplot(incDat, aes(x = BodyTempDiff_jit, y = ExpectedUpper)) +
	geom_vline(xintercept = 0, linetype = 2) +
	
	geom_linerange(aes(ymin = 0, ymax = ExpectedUpper), colour = 'grey60', size = 2, 
								 data = filter(incDat, Lower != Upper)) +
	geom_point(colour = 'grey60', shape = 15) +
	geom_rug(sides = 'b') +
	
	geom_point(aes(y = median, colour = Censoring), data = residsInc) +
	
	scale_color_brewer(palette = 'Dark2') +
	labs(x = 'Body temperature difference', y = 'Left-boundary residual (posterior median)') +
	PLOT_THEME


save_pdf(p1, 'Plots/Supplement_Inc_ResidualsByBodyTempDiff.pdf', width = 5, height = 3)




