##
## Plot excreted virus results:
## 
library(ggplot2)
library(MCMCglmm)
library(ggmcmc)
library(ggstance)

source('Scripts/Utils/utils_mcmcglmm.R')


# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Constants ----------------------------------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
LOG_DOSE_EXPR = "plain(log)[10]~bgroup('(',plain(Dose),')')"

FIXED_EFFECT_NAMES <- tribble(~Parameter,                           ~Label,                                      ~Group,
															'SupertreeDist_std',                   'Phylogenetic distance',                     'Host-virus\ninteraction',
															'logDose_std:SupertreeDist_std',       'Phylogenetic distance x Dose',              'Host-virus\ninteraction',
															
															'BodyTempDiff_std',                    'Body temperature difference',               'Host-virus\ninteraction',
															'logDose_std:BodyTempDiff_std',        'Body temperature difference x Dose',        'Host-virus\ninteraction',
															
															'SourceBodyMass_std',                  'Reservoir body mass',                       'Virus',
															'logDose_std:SourceBodyMass_std',      'Reservoir body mass x Dose',                'Virus',
															
															'DonorIsBat_cen',                      'Bat reservoir',                             'Virus',
															'logDose_std:DonorIsBat_cen',          'Bat reservoir x Dose',                      'Virus',
															
															'InoculatedBodyMass_std',              'Inoculated species body mass',              'Host',
															'logDose_std:InoculatedBodyMass_std',  'Inoculated species body mass x Dose',       'Host',
															
															'InoculatedKnownHost_cen',             'Known reservoir inoculated',                 'Host',
															'logDose_std:InoculatedKnownHost_cen', 'Known reservoir inoculated x Dose',          'Host',
															
															'logDose_std',                         LOG_DOSE_EXPR,                               'Experiment',
															'InoculationPropDist_std',             'Inoculation site distance',                 'Experiment',
															'InoculumTypeBrain_cen',               'Brain-material inoculated',                 'Experiment',
															
															'SourceBodyTemp_std',                  'Reservoir body temperature',                'Body\ntemperature',
															'logDose_std:SourceBodyTemp_std',      'Reservoir body temperature x Dose',         'Body\ntemperature',
															'InoculatedBodyTemp_std',              'Inoculated species body temperature',       'Body\ntemperature',
															'logDose_std:InoculatedBodyTemp_std',  'Inoculated species body temperature x Dose','Body\ntemperature'
)

FIXED_EFFECT_NAMES <- FIXED_EFFECT_NAMES %>% 
	mutate(Label = gsub(' ', '~', Label),
				 Label = if_else(Parameter != 'logDose_std', paste0('plain(', Label, ')'), Label)) %>% 
	mutate(Label = factor(Label, levels = rev(.$Label)),
				 Group = factor(Group, levels = unique(.$Group)))



PLOT_THEME <- theme_bw() +
	theme(strip.background = element_blank(),
				strip.text = element_text(size = 8),
				axis.title = element_text(size = 8),
				axis.text = element_text(size = 7),
				legend.title = element_text(size = 8),
				legend.text = element_text(size = 7),
				panel.grid = element_blank())


# Colours:
SINGLE_COLOUR <- '#8AC2DE'  # Blue
SINGLE_DARK <- '#034e7b'
SINGLE_GREY <- 'grey70'

DEFAULT_BLACK <- '#363636' # Pure black looks too extreme


# Save plots:
save_pdf <- function(p, filename, width, height) {
	pdf(file = filename, width = width, height = height)
	print(p)
	dev.off()
}



# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ----- Load data ---------------------------------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
CorFitPlain <- readRDS('Output/GlandTitres_CorFit_Plain.rds')
CorFitInoc <- readRDS('Output/GlandTitres_CorFit_InocPhylo.rds')
CorFitDose <- readRDS('Output/GlandTitres_CorFit_InocPhylo+Dose.rds')
CorFitFull <- readRDS('Output/GlandTitres_CorFit_AllVars.rds')

dfGlandTitre <- readRDS('Output/dfGlandTitre_includingDerivedVars.rds')


# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ----- Correlation estimates ---------------------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
get_cor <- function(fit, name) {
	posterior <- fit$VCV[, grepl(name, colnames(fit$VCV))] %>% 
		posterior.cor()
	
	Cor <- as.numeric(posterior[, 2])
	HPD <- HPDinterval(as.mcmc(Cor)) %>% 
		data.frame()
	
	cat('Median:', median(Cor), '\nHPD:\n')
	print(HPD)
	
	# Calculate density:
	#  - Need to restrict the density, not the data to allow accurate plotting of HPD
	Density <- data.frame(density(Cor)[c('x', 'y')],
												name = name)
	HPD <- Density %>% 
		filter(x >= HPD$lower, x <= HPD$upper)
	
	# Density
	list(Density = Density, HPD = HPD)
}


## Residual cor:
resCor <- get_cor(CorFitPlain, 'units')

p <- ggplot(resCor$Density, aes(x = x, y = y)) +
	geom_area(colour = NA, fill = SINGLE_COLOUR) +
	geom_area(colour = NA, fill = SINGLE_DARK, data = resCor$HPD) +
	geom_vline(xintercept = 0, linetype = 2) +
	labs(x = 'Residual correlation', y = 'Density') + 
	coord_cartesian(xlim = c(-1, 1)) +
	PLOT_THEME

save_pdf(p, 'Plots/Fig3_GlandTitre_CorPlain.pdf', width = 2.5, height = 1.5)




## Residual and Inoculated Spp cor:
resCor <- get_cor(CorFitInoc, 'units')


p <- ggplot(resCor$Density, aes(x = x, y = y)) +
	geom_area(colour = NA, fill = SINGLE_COLOUR) +
	geom_area(colour = NA, fill = SINGLE_DARK, data = resCor$HPD) +
	geom_vline(xintercept = 0, linetype = 2) +
	labs(x = 'Residual correlation', y = 'Density') +
	coord_cartesian(xlim = c(-1, 1)) +
	PLOT_THEME

save_pdf(p, 'Plots/Fig3_GlandTitre_CorInoc.pdf', width = 2.5, height = 1.5)



## Correlations when dose is added:
resCor <- get_cor(CorFitDose, 'units')

p <- ggplot(resCor$Density, aes(x = x, y = y)) +
	geom_area(colour = NA, fill = SINGLE_COLOUR) +
	geom_area(colour = NA, fill = SINGLE_DARK, data = resCor$HPD) +
	geom_vline(xintercept = 0, linetype = 2) +
	labs(x = 'Residual correlation', y = 'Density') +
	coord_cartesian(xlim = c(-1, 1)) +
	PLOT_THEME

save_pdf(p, 'Plots/Fig3_GlandTitre_CorInoc+Dose.pdf', width = 2.5, height = 1.5)



## Correlations in full model:
resCor <- get_cor(CorFitFull, 'units')

p <- ggplot(resCor$Density, aes(x = x, y = y)) +
	geom_area(colour = NA, fill = SINGLE_COLOUR) +
	geom_area(colour = NA, fill = SINGLE_DARK, data = resCor$HPD) +
	geom_vline(xintercept = 0, linetype = 2) +
	labs(x = 'Residual correlation', y = 'Density') +
	coord_cartesian(xlim = c(-1, 1)) +
	PLOT_THEME

save_pdf(p, 'Plots/Fig3_GlandTitre_CorFullModel.pdf', width = 1.8, height = 1.5)


# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ----- Coefficient estimates ---------------------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
get_coefs <- function(fit, traitNames) {
	Coefs <- fit$Sol[, colnames(fit$X)] %>%   # Get fixed effect posterior
		as.mcmc.list()
	
	CoefSummary <- summarise_posterior(Coefs, chainsSeparate = FALSE) %>% 
		separate(Parameter, into = c('Trait', 'Parameter'), sep = ':', extra = 'merge') %>% 
		filter(!is.na(Parameter)) %>%   # Remove intercept
		mutate(Significant = !(lower < 0 & upper > 0)) %>% 
		left_join(FIXED_EFFECT_NAMES, by = 'Parameter')
	
	CoefSummary$TraitName <- traitNames[CoefSummary$Trait]
	
	CoefDistribution <- ggs(Coefs) %>% 
		separate(Parameter, into = c('Trait', 'Parameter'), sep = ':', extra = 'merge') %>% 
		filter(!is.na(Parameter)) %>% 
		left_join(CoefSummary, by = c('Trait', 'Parameter'))
	
	# Return:
	list(Summary = CoefSummary,
			 Distribution = CoefDistribution)
}


facetNames <- c(traitinc = 'Incubation period',
								traitGlandTitre = 'Salivary gland titre')

Coefs <- get_coefs(CorFitFull, facetNames)

Coefs$Summary <- Coefs$Summary %>% 
	filter(TraitName == 'Salivary gland titre')

Coefs$Distribution <- Coefs$Distribution %>% 
	filter(TraitName == 'Salivary gland titre')
	
p <- ggplot(Coefs$Summary, aes(y = Label, x = median, colour = Significant, fill = Significant)) +
	geom_vline(xintercept = 0, linetype = 2) +
	geom_violinh(aes(x = value), colour = NA, data = Coefs$Distribution) +
	geom_linerangeh(aes(xmin = lower, xmax = upper)) +
	geom_point() +
	scale_color_manual(values = c('TRUE' = DEFAULT_BLACK, 'FALSE' = 'grey30'), guide = F) +
	scale_fill_manual(values = c('TRUE' = SINGLE_COLOUR, 'FALSE' = SINGLE_GREY), guide = F) +
	scale_y_discrete(name = NULL, label = function(x) parse(text = x)) +
	facet_grid(Group ~ ., scales = 'free', shrink = T) +
	labs(x = 'Effect size') +
	PLOT_THEME


save_pdf(p, 'Plots/Fig3_GlandTitre_Coefs.pdf', width = 3.75, height = 3)



# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ----- Predicted effect: Known host x Dose -----------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
probs <- c(0.125, 0.25, 0.5, 0.75)
DoseQs <- quantile(log10(dfGlandTitre$Dose), probs = probs) %>% 
	standardise_gelman(Mean = mean(log10(dfGlandTitre$Dose)), SD = sd(log10(dfGlandTitre$Dose)))

names(probs) <- as.character(DoseQs)

# Predict
modelData <- dfGlandTitre %>% 
	select(logDose_std, InoculationPropDist_std, InoculatedKnownHost_cen, BodyTempDiff_std, ExperimentID, InoculatedSpecies)

preds <- predict_effect_joint(CorFitFull, modelData, traits = c('GlandTitre', 'inc'),
															var1 = 'InoculatedKnownHost_cen', var2 = 'logDose_std',
															var1Values = c(min(dfGlandTitre$InoculatedKnownHost_cen), max(dfGlandTitre$InoculatedKnownHost_cen)),
															var2Values = DoseQs,
															returnPosterior = TRUE)

preds$pred <- preds$pred %>% 
	mutate(InoculatedKnownHost = rev_center_only(InoculatedKnownHost_cen, dfGlandTitre$InoculatedKnownHost),
				 logDose = rev_standardise_gelman(logDose_std, log10(dfGlandTitre$Dose)),
				 Quartile = probs[as.character(logDose_std)],
				 DoseLabel = sprintf('Q[%f]==%.2f', Quartile, logDose)) %>% 
	filter(trait == 'GlandTitre') %>% 
	mutate(InoculatedKnownHost = if_else(InoculatedKnownHost == 1, 'True', 'False'))

preds$posterior <- preds$posterior %>% 
	mutate(InoculatedKnownHost = rev_center_only(InoculatedKnownHost_cen, dfGlandTitre$InoculatedKnownHost),
				 logDose = rev_standardise_gelman(logDose_std, log10(dfGlandTitre$Dose)),
				 Quartile = probs[as.character(logDose_std)],
				 DoseLabel = sprintf('Q[%f]==%.2f', Quartile, logDose)) %>% 
	filter(trait == 'GlandTitre') %>% 
	mutate(InoculatedKnownHost = if_else(InoculatedKnownHost == 1, 'True', 'False'))

# Plot:
p1 <- ggplot(preds$pred, aes(x = InoculatedKnownHost, y = Median)) +
	geom_violin(aes(y = PredictedMean, group = InoculatedKnownHost), data = preds$posterior, fill = SINGLE_COLOUR, colour = NA) +
	
	geom_point(colour = SINGLE_DARK, size = 2) +
	geom_linerange(aes(ymin = Lower, ymax = Upper), colour = SINGLE_DARK) +
	geom_line(aes(group = DoseLabel), linetype = '12', colour = DEFAULT_BLACK) +
	
	
	facet_grid(. ~ DoseLabel, labeller = label_parsed) +
	labs(x = 'Known reservoir inoculated', y = expression(log(1 + plain(Salivary~gland~titre)))) +
	coord_cartesian(ylim = c(-10, 20)) +
	PLOT_THEME


save_pdf(p1, 'Plots/Fig3_GlandTitre_KnownHostXDose_Grid.pdf', width = 3.75, height = 1.7)


# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ----- Predicted effect: Body temp diff x Dose -------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
doseLims <- quantile(log10(dfGlandTitre$Dose), c(0.05, 0.95))
tempDiffLims <- quantile(dfGlandTitre$BodyTempDiff, c(0.05, 0.95))

predTemps <- seq(min(dfGlandTitre$BodyTempDiff_std), max(dfGlandTitre$BodyTempDiff_std), by = 0.1)
predDoses <- seq(min(dfGlandTitre$logDose_std), max(dfGlandTitre$logDose_std), by = 0.1)

preds <- predict_effect_joint(CorFitFull, modelData, traits = c('GlandTitre', 'inc'),
															var1 = 'BodyTempDiff_std', var2 = 'logDose_std',
															var1Values = predTemps,
															var2Values = predDoses,
															returnPosterior = FALSE)

preds <- preds %>% 
	mutate(logDose = rev_standardise_gelman(logDose_std, log10(dfGlandTitre$Dose)),
				 BodyTempDiff = rev_standardise_gelman(BodyTempDiff_std, dfGlandTitre$BodyTempDiff)) %>% 
	filter(trait == 'GlandTitre')


colourLims <- preds %>% 
	filter(logDose >= doseLims[1] & logDose <= doseLims[2]) %>% 
	filter(BodyTempDiff >= tempDiffLims[1]-0.2 & BodyTempDiff <= tempDiffLims[2]) %>% 
	.$PredictedMean %>% 
	range()

	
p1 <- ggplot(preds, aes(x = BodyTempDiff, y = logDose)) +
	geom_raster(aes(fill = PredictedMean)) +
	geom_rug(aes(y = log10(Dose)), alpha = 0.6, data = dfGlandTitre,
					 position = position_jitter(width = 0.01, height = 0.05)) +
	geom_vline(xintercept = 0, linetype = 2) +
	
	scale_fill_viridis_c(direction = -1, limits = colourLims) +
	
	labs(x = 'Body temperature difference (°C)', y = parse(text = LOG_DOSE_EXPR), 
			 fill = expression(log(1+Salivary~gland~titre))) +
	coord_cartesian(xlim = tempDiffLims, ylim = doseLims, expand = FALSE) +
	guides(fill = guide_colourbar(label.position = 'top')) +
	PLOT_THEME +
	theme(legend.position = 'top',
				legend.key.height = unit(0.6, 'lines'),
				legend.title = element_text(vjust = 0.13),
				legend.margin = margin(b = 0.4, unit = 'lines'))


save_pdf(p1, 'Plots/Fig3_GlandTitre_BodyTempDiffXDose_Grid.pdf', width = 3.75, height = 2)
