## Util functions for MCMCglmm fits
require(dplyr)
require(tidyr)
require(purrr)
require(coda)

# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Summarise posterior distributions ----------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

get_posterior_median <- function(samples, chainsSeparate = T) {
	.get_median <- function(samples) {
		postSummary <- summary(samples)
		postSummary$quantiles[, '50%']
	}
	
	
	if (chainsSeparate) return(lapply(samples, .get_median))
	.get_median(samples)
}

get_hpd_interval <- function(samples, chainsSeparate = T, prob = 0.95) {
	if (chainsSeparate) return(HPDinterval(samples, prob = prob))
	
	# Collapse/merge chains before calculating HPD
	as.matrix(samples) %>% 
		as.mcmc() %>% 
		HPDinterval(prob = prob)
}


## Combined summary:
summarise_posterior <- function(samples, chainsSeparate = T) {
	meds <- get_posterior_median(samples, chainsSeparate = chainsSeparate)
	hpd <- get_hpd_interval(samples, chainsSeparate = chainsSeparate)
	
	if (chainsSeparate) {
		result <- lapply(1:length(samples), function(i) data.frame(Chain = i,
																															 Parameter = colnames(samples[[i]]),
																															 median = meds[[i]],
																															 hpd[[i]])) %>% 
			bind_rows()
	} else {
		result <- data.frame(Parameter = colnames(samples[[1]]),
												 median = meds,
												 hpd)
	}
	
	result
}


# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Data transformation ------------------------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
standardise_gelman <- function(vec, Mean = NULL, SD = NULL) {
	## Standardise a variable to have mean 0 and sd of 0.5, as in Gelman2008
	## Handles NA's by ignoring them, but with a warning (since we should really be
	#   imputing before standardising...)
	#  If Mean or SD are not supplied, they will be calculated from vec
	if (any(is.na(vec))) {
		warning("NA's found in variable to be standardised - consider standardising inside model instead")
		completeVec <- na.omit(vec)
	} else {
		completeVec <- vec
	}
	
	if(is.null(Mean)) Mean <- mean(completeVec)
	if(is.null(SD)) SD <- sd(completeVec)
	
	(vec - Mean) / (2*SD)
}


# Reverse standardisation:
rev_standardise_gelman <- function(stdVals, originalVals) {
	# Standardisation used: (vec - Mean) / (2*SD)
	origMean <- mean(originalVals)
	origSD <- sd(originalVals)
	
	(stdVals * (2*origSD)) + origMean
}


center_only <- center_only <- function(x) {
	# Center binary vars as recommended in Gelman2008 :
	x - mean(x)
}

rev_center_only <- function(cenVals, originalVals) {
	# Reverse centering
	cenVals + mean(originalVals)
}


# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Prediction ------------------------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

predict_marginal_effect <- function(fit, var1, var2 = NULL, resolution = 0.1, var1Values = NULL, var2Values = NULL) {
	# Predict the marginal effect of a variable. For interactions, supply var2 as well
	# - By default all combinations of the two variables will be predicted, unless values
	#   are explicitly supplied in either var1Values, var2Values, or both.
	# NOTE: the interval returned is the confidence interval, which marginalises over the 
	# residual error distribution (in addition to marginalising over random effect 
	# distributions). This represents the confidence interval around the mean prediction, and
	# will be narrower than the 'prediction interval' which takes residual error into account
	
	# Extract data from fit:
	modelData <- fit$X %>% 
		as.matrix() %>% 
		as.data.frame() %>% 
		select(-matches('(Intercept)')) %>% 
		select(-contains(':'))  # Remove interaction columns
	
	reData <- fit$Z %>% 
		as.matrix() %>% 
		as.data.frame() %>% 
		gather('key', 'value') %>% 
		filter(value == 1) %>% 
		separate(key, into = c('ColName', 'ColVal'), sep = '\\.') %>% 
		group_by(ColName) %>% 
		mutate(RowID = 1:n()) %>% 
		ungroup() %>% 
		spread(ColName, ColVal) %>% 
		select(-value, -RowID)
	
	modelData <- bind_cols(modelData, reData)
	
	# Extract response variable name(s): 
	#  - These need to be present in the newdata passed to predict, though the values don't matter
	#  - Extraction is complicated by the fact that the response side of the formula may contain
	#    an expression and more than one variable
	modelVars <- all.vars(fit$Fixed$formula)
	responseString <- as.character(fit$Fixed$formula)[[2]]
	responseVars <- sapply(modelVars, grepl, x = responseString)
	responseVars <- modelVars[responseVars]
	
	responseData <- matrix(0, ncol = length(responseVars)) %>% data.frame()
	colnames(responseData) <- responseVars
	
	
	# Predict data: use means for numeric columns, most common level for categorical (though for RE's this does not matter)
	numericCols <- modelData %>% 
		summarise_if(is.numeric, mean)
	
	categoricalCols <- modelData %>% 
		summarise_if(negate(is.numeric),
								 ~names(sort(table(.),decreasing=TRUE)[1]))
	
	
	# Add a smooth range of values for the key variables(s)
	if (is.null(var1Values)) {
		var1Values <- seq(min(modelData[, var1]), max(modelData[, var1]), by = resolution)
	}
	
	if(! is.null(var2)) {
		# If an interaction, get all possible combinations of the two variables:
		if (is.null(var2Values)) 
			var2Values <- seq(min(modelData[, var2]), max(modelData[, var2]), by = resolution)
		
		predData <- data.frame(expand.grid(var1Values, var2Values))
		colnames(predData) <- c(var1, var2)
		
	} else {
		predData <- data.frame(var1Values)
		colnames(predData) <- var1
	}
	
	predData <- bind_cols(responseData, numericCols, categoricalCols) %>% 
		select(-one_of(var1, var2)) %>% 
		data.frame(., predData)
	
	
	# Get marginal predictions and return:
	predict.MCMCglmm(fit, newdata = predData, type = 'response', interval = 'confidence') %>% 
		data.frame() %>% 
		bind_cols(predData)
}


predict_marginal_distribution <- function(fit, var1, var2 = NULL, resolution = 0.1, var1Values = NULL, var2Values = NULL) {
	# Predict the marginal effect of a variable, returning the posterior distribution at various values
	# For interactions, supply var2 as well.
	# NOTE: as in predict_marginal_effect(), this function marginalises over the 
	# residual error distribution (in addition to marginalising over random effect 
	# distributions). The result represents the posterior distribution of the prediction
	# (or rather the posterior distribution of the mean predicted effect??)
	
	# Extract data from fit:
	modelData <- fit$X %>% 
		as.matrix() %>% 
		as.data.frame() %>% 
		select(-contains(':'))  # Remove interaction columns
	
	reData <- fit$Z %>% 
		as.matrix() %>% 
		as.data.frame() %>% 
		gather('key', 'value') %>% 
		filter(value == 1) %>% 
		separate(key, into = c('ColName', 'ColVal'), sep = '\\.') %>% 
		group_by(ColName) %>% 
		mutate(RowID = 1:n()) %>% 
		ungroup() %>% 
		spread(ColName, ColVal) %>% 
		select(-value, -RowID)
	
	modelData <- bind_cols(modelData, reData)
	
	# Extract response variable name(s): 
	#  - These need to be present in the newdata passed to predict, though the values don't matter
	#  - Extraction is complicated by the fact that the response side of the formula may contain
	#    an expression and more than one variable
	modelVars <- all.vars(fit$Fixed$formula)
	responseString <- as.character(fit$Fixed$formula)[[2]]
	responseVars <- sapply(modelVars, grepl, x = responseString)
	responseVars <- modelVars[responseVars]
	
	responseData <- matrix(0, ncol = length(responseVars)) %>% data.frame()
	colnames(responseData) <- responseVars
	
	
	# Predict data: use means for numeric columns, most common level for categorical (though for RE's this does not matter)
	numericCols <- modelData %>% 
		summarise_if(is.numeric, mean)
	
	categoricalCols <- modelData %>% 
		summarise_if(negate(is.numeric),
								 ~names(sort(table(.),decreasing=TRUE)[1]))
	
	
	# Combine with the supplied values
	if(! is.null(var2)) {
		# If an interaction, get all possible combinations of the two variables supplied:
		predData <- data.frame(expand.grid(var1Values, var2Values))
		colnames(predData) <- c(var1, var2)
		
	} else {
		predData <- data.frame(var1Values)
		colnames(predData) <- var1
	}
	
	predData <- bind_cols(responseData, numericCols, categoricalCols) %>% 
		select(-one_of(var1, var2)) %>% 
		data.frame(., predData)
	
	
	# Get marginal predictions and return:
	get_prediction <- function(iteration) {
		cat(paste(iteration, '\r'))
		predict.MCMCglmm(fit, newdata = predData, type = 'response', it = iteration) %>% 
			data.frame(fit = .) %>% 
			bind_cols(predData)
	}
	
	Niterations <- nrow(fit$Sol)  # Number of posterior samples available
	
	cat(paste0('Predicting\n      / ', Niterations, '\r'))
	lapply(1:Niterations, get_prediction) %>% 
		bind_rows(.id = 'Iteration')
}




# ------------------------------
### FOR JOINT MODELS:

predict_effect_joint <- function(fit, modelData, traits, var1, var2, var1Values, var2Values, returnPosterior = FALSE) {
	# Predictions for multi-response models:
	# - Can't get predict.mcmcglmm to work on new data in this case, so calculate predictions manually
	# - NOTE: setting returnPosterior = TRUE should only be done with a small number of var1 and var2 values...
	
	# Extract response variable name(s): 
	#  - These need to be present in the newdata passed to predict, though the values don't matter
	modelVars <- all.vars(fit$Fixed$formula)
	responseString <- as.character(fit$Fixed$formula)[[2]]
	responseVars <- sapply(modelVars, grepl, x = responseString)
	responseVars <- modelVars[responseVars]
	
	responseData <- matrix(0, ncol = length(responseVars)) %>% data.frame()
	colnames(responseData) <- responseVars
	
	# New predict data: use means for numeric columns, most common level for categorical (though for RE's this does not matter)
	numericCols <- modelData %>% 
		summarise_if(is.numeric, mean)
	
	categoricalCols <- modelData %>% 
		summarise_if(negate(is.numeric),
								 ~names(sort(table(.),decreasing=TRUE)[1]))
	
	# Add all possible combinations of var 1 and 2 values supplied:
	predData <- data.frame(expand.grid(var1Values, var2Values))
	colnames(predData) <- c(var1, var2)
	
	predData <- bind_cols(responseData, numericCols, categoricalCols) %>% 
		select(-one_of(var1, var2)) %>% 
		data.frame(., predData)
	
	# Add traits by duplicating dataframe for each trait:
	predData <- lapply(traits, function(x) predData)
	names(predData) <- traits
	predData <- bind_rows(predData, .id = 'trait')
	
	# Expand to match the model matrix:
	predMatrix <- model.matrix(fit$Fixed$formula, data = predData)
	predMatrix <- predMatrix[, colnames(fit$Sol)]  # re-arrange to match internal order
	
	
	
	# Predictions:
	if (!returnPosterior) {
		# Get posterior mode only, then exit early:
		pred <- predMatrix %*% posterior.mode(fit$Sol)
		return(cbind(predData, PredictedMean = pred))
	}
	
	# Otherwise, get full posterior distribution of (marginal) predictions:
	.get_pred <- function(iteration) {
		pred <- predMatrix %*% fit$Sol[iteration, ]
		pred <- drop(pred)
		
		cbind(predData, PredictedMean = pred)
	}
	posteriorPreds <- lapply(1:nrow(fit$Sol), .get_pred) %>% 
		bind_rows()
	
	# Get Mean and HPD:
	posteriorSummary <- posteriorPreds %>% 
		group_by_('trait', var1, var2) %>% 
		summarise(Median = median(PredictedMean),
							Lower = HPDinterval(as.mcmc(PredictedMean))[, 'lower'],
							Upper = HPDinterval(as.mcmc(PredictedMean))[, 'upper'])
	
	# Return:
	list(pred = posteriorSummary, posterior = posteriorPreds)
}


# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# ---- Residuals ------------------------------------------------------------------------
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

## Calculate the linear term (conditional prediction) at a given iteration
calculate_linear_term <- function(fit, iteration) {
	designMat <- cBind(fit$X, fit$Z)
	coefs <- fit$Sol[iteration, ]
	
	if (length(coefs) != ncol(designMat)) stop('Random effect posterior distribution not available. Specify pr = TRUE when running MCMCglmm')
	
	pred <- designMat %*% coefs
	as.numeric(pred)
}



## Summarise the posterior distribution left-bound residuals
##  - The left boundary is always known, so can be used to construct a sort of residual
get_leftbound_residuals <- function(chains, obsLower, obsUpper) {
	nsamples <- nrow(chains[[1]]$Sol)  # Assume all chains were equal lenght
	posteriorResult <- list()
	
	for (fit in chains) {
		# Calculate posterior residuals from posterior distribution of fitted vals
		fittedVals <- lapply(1:nsamples, calculate_linear_term, fit = fit)
		
		residsPosterior <- lapply(fittedVals, function(fitted) obsLower - as.numeric(fitted))
		names(residsPosterior) <- as.character(1:nsamples)
		residsPosterior <- bind_rows(residsPosterior)
		residsPosterior <- t(residsPosterior)
		
		posteriorResult <- c(posteriorResult,
												 list(as.mcmc(residsPosterior)))
	}
	
	posteriorResult <- mcmc.list(posteriorResult)
	
	# Summarise result and return
	median <- get_posterior_median(posteriorResult)
	hpd <- get_hpd_interval(posteriorResult)
	
	nchains <- length(posteriorResult)
	res <- lapply(1:nchains, function(i) data.frame(ObsID = 1:length(obsLower), 
																									ObsLower = obsLower,
																									ObsUpper = obsUpper,
																									median = median[[i]], 
																									hpd[[i]]))
	names(res) <- as.character(1:nchains)
	
	bind_rows(res, .id = 'Chain')
}



#--------------------------------------------------------------------------------------------------#




