## Utility functions for survival data


split_sub_species <- function(speciesvec) {
	# Splits a species name into species and sub-species, if possible
	# Returns a data frame with two columns
	splitvec <- strsplit(speciesvec, " ")
	res <- data.frame()
	
	for (split in splitvec) {
		if (length(split) == 2) {
			species <- paste(split, collapse = " ")
			res <- rbind(res, data.frame(species, subspecies="", stringsAsFactors = F))
		} else if (length(split) == 3) {
			species <- paste(split[1:2], collapse = " ")
			res <- rbind(res, data.frame(species, subspecies=split[3], stringsAsFactors = F))
		} else {
			warning("More than 3 names found")
		}
	}
	
	res
}

get_dist <- function(Spp1, Spp2, dist_matrix) { 
	# Return a result from dist_matrix, returning NA when either of the species are not present
	stopifnot(length(Spp1) == length(Spp2))
	result <- vector()
	
	for (i in 1:length(Spp1)) {
		firstSpp <- Spp1[i]
		secondSpp <- Spp2[i]
		
		if (firstSpp %in% colnames(dist_matrix) & secondSpp %in% colnames(dist_matrix)) {
			result <- c(result, dist_matrix[firstSpp, secondSpp])
		} else {
			result <- c(result, NA)
		}
	}
	return(result)
}

convert_to_survtime <- function(censorCode, jagsLower, jagsUpper) {
	# Converts jags censor limits to survreg-compatible timings
	stopifnot( length(censorCode) == length(jagsLower) & length(jagsLower) == length(jagsUpper) )
	
	survLower <- vector(mode = "double", length = length(censorCode))
	survUpper <- vector(mode = "double", length = length(censorCode))
	
	for (i in 1:length(censorCode)) {
		if (censorCode[i] == 0) {
			survLower[i] <- jagsLower[i]
			survUpper[i] <- jagsLower[i]
			
		} else if (censorCode[i] == 1) {
			survLower[i] <- jagsLower[i]
			survUpper[i] <- jagsUpper[i]
			
		} else if (censorCode[i] == 2) {
			survLower[i] <- jagsUpper[i]
			survUpper[i] <- NA
			
		} else {
			message <- paste0("Invalid censorCode at index ", i, ": ", censorCode[i])
			stop(message)
		}
	}
	
	list(Lower = survLower, Upper = survUpper)
}

convert_to_nimbletime <- function(censorCode, jagsLower, jagsUpper) {
	## Converts jags censor limits to ones compatible with my custom nimble samplers:
	#   The only difference is in the handling of right-censoring - JAGS uses the "upper"
	#   value as the lower limit for right censoring, while dsurvmix uses the lower limit
	#   in this case. In addition, sampler_rw_mix uses both limits to ensure the random
	#   walk stays within the known bounds (if reflective sampling is turned on), so we
	#   need to supply Inf as the upper limit for right-censored observations
	stopifnot( length(censorCode) == length(jagsLower) & length(jagsLower) == length(jagsUpper) )
	
	nimLower <- vector(mode = "double", length = length(censorCode))
	nimUpper <- vector(mode = "double", length = length(censorCode))
	
	for (i in 1:length(censorCode)) {
		if (censorCode[i] == 0) {
			nimLower[i] <- jagsLower[i]  # same as in convert_to_survtime(), but limits don't 
			nimUpper[i] <- jagsLower[i]  # actually matter here, since values won't be sampled
			
		} else if (censorCode[i] == 1) {
			nimLower[i] <- jagsLower[i]  # same as in convert_to_survtime()
			nimUpper[i] <- jagsUpper[i]
			
		} else if (censorCode[i] == 2) {
			nimLower[i] <- jagsUpper[i]  # same as in convert_to_survtime(), except 
			nimUpper[i] <- Inf           # NA is changed to Inf
			
		} else {
			message <- paste0("Invalid censorCode at index ", i, ": ", censorCode[i])
			stop(message)
		}
	}
	
	list(Lower = nimLower, Upper = nimUpper)
}

nimtime_to_survtime <- function(censorCode, nimLower, nimUpper) {
	# Converts nimble censor limits to survreg-compatible timings
	stopifnot( length(censorCode) == length(nimLower) & length(nimLower) == length(nimUpper) )
	
	survLower <- vector(mode = "double", length = length(censorCode))
	survUpper <- vector(mode = "double", length = length(censorCode))
	
	for (i in 1:length(censorCode)) {
		if (is.na(censorCode[i]) & is.na(nimLower[i]) & is.na(nimUpper[i])) {
			survLower[i] <- NA
			survUpper[i] <- NA
			
		} else if (censorCode[i] == 0) {
			survLower[i] <- nimLower[i]
			survUpper[i] <- nimLower[i]
			
		} else if (censorCode[i] == 1) {
			survLower[i] <- nimLower[i]
			survUpper[i] <- nimUpper[i]
			
		} else if (censorCode[i] == 2) {
			survLower[i] <- nimLower[i]
			survUpper[i] <- NA  # inf in nimble times
			
		} else {
			message <- paste0("Invalid censorCode at index ", i, ": ", censorCode[i])
			stop(message)
		}
	}
	
	list(Lower = survLower, Upper = survUpper)
}

nimtime_to_jagstime <- function(censorCode, nimLower, nimUpper) {
	# Converts nimble censor limits to survreg-compatible timings
	stopifnot( length(censorCode) == length(nimLower) & length(nimLower) == length(nimUpper) )
	
	jagsLower <- vector(mode = "double", length = length(censorCode))
	jagsUpper <- vector(mode = "double", length = length(censorCode))
	
	for (i in 1:length(censorCode)) {
		if (is.na(censorCode[i]) & is.na(nimLower[i]) & is.na(nimUpper[i])) {
			jagsLower[i] <- NA
			jagsUpper[i] <- NA
			
		} else if (censorCode[i] == 0) {
			jagsLower[i] <- nimLower[i]
			jagsUpper[i] <- nimLower[i]
			
		} else if (censorCode[i] == 1) {
			jagsLower[i] <- nimLower[i]
			jagsUpper[i] <- nimUpper[i]
			
		} else if (censorCode[i] == 2) {
			jagsLower[i] <- NA
			jagsUpper[i] <- nimLower[i]  
			
		} else {
			message <- paste0("Invalid censorCode at index ", i, ": ", censorCode[i])
			stop(message)
		}
	}
	
	list(Lower = jagsLower, Upper = jagsUpper)
}